import os
import unittest
import warnings

from robot.utils.argumentparser import ArgumentParser
from robot.utils.asserts import (assert_equal, assert_raises,
                                 assert_raises_with_msg, assert_true)
from robot.errors import Information, DataError, FrameworkError
from robot.version import get_full_version


USAGE = """Example Tool -- Stuff before hyphens is considered name

Usage:  robot.py [options] datafile

Version: <VERSION>

Options:
  -d --reportdir dir        Explanation
  -r --reportfile file      This explanation continues ............... 78
       ........... to multiple lines.
       Next line is totally empty.

  -E --escape what:with *      Line below has nothing after '*'. Next line has
          nothing after value and next nothing after option name
  -v --variable name:value *
  -N --name name
  -t -T --toggle   Something
  -h -? --help
  --version  Explanation

  -z   No long option so not an option line.
  --z  No long option here either
  this line doesn't start with a '-' so not an --optionline
  -\\-option     escaped 1
  -o -\\-option  escaped 2
          --ignored  options cannot be this far
          --ignored

* denotes options that can be set multiple times
"""

USAGE2 = """Just Name Here
usage:  robot.py [options] arg1 arg2

options:
  -v --variable name=value
  -x --var-able name=v1,v2   Explanation
  -3 --42
  --help
"""


class TestArgumentParserInit(unittest.TestCase):

    def setUp(self):
        self.ap = ArgumentParser(USAGE)

    def assert_long_opts(self, expected, ap=None):
        expected += ['no' + e for e in expected if not e.endswith('=')]
        long_opts = (ap or self.ap)._long_opts
        assert_equal(sorted(long_opts), sorted(expected))

    def assert_short_opts(self, expected, ap=None):
        assert_equal((ap or self.ap)._short_opts, expected)

    def assert_multi_opts(self, expected, ap=None):
        assert_equal((ap or self.ap)._multi_opts, expected)

    def assert_flag_opts(self, expected, ap=None):
        assert_equal((ap or self.ap)._flag_opts, expected)

    def test_short_options(self):
        self.assert_short_opts('d:r:E:v:N:tTh?')

    def test_long_options(self):
        self.assert_long_opts(['reportdir=', 'reportfile=', 'escape=',
                               'variable=', 'name=', 'toggle', 'help',
                               'version'])

    def test_multi_options(self):
        self.assert_multi_opts(['escape', 'variable'])

    def test_flag_options(self):
        self.assert_flag_opts(['toggle', 'help', 'version'])

    def test_options_must_be_indented_by_1_to_four_spaces(self):
        ap = ArgumentParser('''Name
1234567890
--notin  this option is not indented at all and thus ignored
 --opt1
    --opt2        This option is 4 spaces from left -> included
    -o --opt3 argument  It doesn't matter how far the option gets.
     --notopt     This option is 5 spaces from left -> not included
     -i --ignored
                     --not-in-either
    --included  back in four space indentation''')
        self.assert_long_opts(['opt1', 'opt2', 'opt3=', 'included'], ap)

    def test_case_insensitive_long_options(self):
        ap = ArgumentParser(' -f --foo\n -B --BAR\n')
        self.assert_short_opts('fB', ap)
        self.assert_long_opts(['foo', 'bar'], ap)

    def test_long_options_with_hyphens(self):
        ap = ArgumentParser(' -f --f-o-o\n -B --bar--\n')
        self.assert_short_opts('fB', ap)
        self.assert_long_opts(['foo', 'bar'], ap)

    def test_same_option_multiple_times(self):
        for usage in [' --foo\n --foo\n',
                      ' --foo\n -f --Foo\n',
                      ' -x --foo xxx\n -y --Foo yyy\n',
                      ' -f --foo\n -f --bar\n']:
            assert_raises(FrameworkError, ArgumentParser, usage)
        ap = ArgumentParser(' -f --foo\n -F --bar\n')
        self.assert_short_opts('fF', ap)
        self.assert_long_opts(['foo', 'bar'], ap)

    def test_same_option_multiple_times_with_no_prefix(self):
        for usage in [' --foo\n --nofoo\n',
                      ' --nofoo\n --foo\n'
                      ' --nose size\n --se\n']:
            assert_raises(FrameworkError, ArgumentParser, usage)
        ap = ArgumentParser(' --foo value\n --nofoo value\n')
        self.assert_long_opts(['foo=', 'nofoo='], ap)


class TestArgumentParserParseArgs(unittest.TestCase):

    def setUp(self):
        self.ap = ArgumentParser(USAGE)

    def test_missing_argument_file_throws_data_error(self):
        inargs = '--argumentfile missing_argument_file_that_really_is_not_there.txt'.split()
        self.assertRaises(DataError, self.ap.parse_args, inargs)

    def test_single_options(self):
        inargs = '-d reports --reportfile reps.html -T arg'.split()
        opts, args = self.ap.parse_args(inargs)
        assert_equal(opts, {'reportdir': 'reports', 'reportfile': 'reps.html',
                            'escape': [], 'variable': [], 'name': None,
                            'toggle': True})

    def test_multi_options(self):
        inargs = '-v a:1 -v b:2 --name my_name --variable c:3 arg'.split()
        opts, args = self.ap.parse_args(inargs)
        assert_equal(opts, {'variable': ['a:1', 'b:2', 'c:3'], 'escape': [],
                            'name': 'my_name', 'reportdir': None,
                            'reportfile': None, 'toggle': None})
        assert_equal(args, ['arg'])

    def test_flag_options(self):
        for inargs, exp in [('', None),
                            ('--name whatever', None),
                            ('--toggle', True),
                            ('-T', True),
                            ('--toggle --name whatever -t', True),
                            ('-t -T --toggle', True),
                            ('--notoggle', False),
                            ('--notoggle --name xxx --notoggle', False),
                            ('--toggle --notoggle', False),
                            ('-t -t -T -T --toggle -T --notoggle', False),
                            ('--notoggle --toggle --notoggle', False),
                            ('--notoggle --toggle', True),
                            ('--notoggle --notoggle -T', True)]:
            opts, args = self.ap.parse_args(inargs.split() + ['arg'])
            assert_equal(opts['toggle'], exp, inargs)
            assert_equal(args, ['arg'])

    def test_flag_option_with_no_prefix(self):
        ap = ArgumentParser(' -S --nostatusrc\n --name name')
        for inargs, exp in [('', None),
                            ('--name whatever', None),
                            ('--nostatusrc', False),
                            ('-S', False),
                            ('--nostatusrc -S --nostatusrc -S -S', False),
                            ('--statusrc', True),
                            ('--statusrc --statusrc -S', False),
                            ('--nostatusrc --nostatusrc -S --statusrc', True)]:
            opts, args = ap.parse_args(inargs.split() + ['arg'])
            assert_equal(opts['statusrc'], exp, inargs)
            assert_equal(args, ['arg'])

    def test_single_option_multiple_times(self):
        for inargs in ['--name Foo -N Bar arg',
                       '-N Zap --name Foo --name Bar arg',
                       '-N 1 -N 2 -N 3 -t --variable foo -N 4 --name Bar arg']:
            opts, args = self.ap.parse_args(inargs.split())
            assert_equal(opts['name'], 'Bar')
            assert_equal(args, ['arg'])

    def test_case_insensitive_long_options(self):
        opts, args = self.ap.parse_args('--VarIable X:y --TOGGLE arg'.split())
        assert_equal(opts['variable'], ['X:y'])
        assert_equal(opts['toggle'], True)
        assert_equal(args, ['arg'])

    def test_case_insensitive_long_options_with_equal_sign(self):
        opts, args = self.ap.parse_args('--VariAble=X:y --VARIABLE=ZzZ'.split())
        assert_equal(opts['variable'], ['X:y', 'ZzZ'])
        assert_equal(args, [])

    def test_long_options_with_hyphens(self):
        opts, args = self.ap.parse_args('--var-i-a--ble x-y ----toggle---- arg'.split())
        assert_equal(opts['variable'], ['x-y'])
        assert_equal(opts['toggle'], True)
        assert_equal(args, ['arg'])

    def test_long_options_with_hyphens_with_equal_sign(self):
        opts, args = self.ap.parse_args('--var-i-a--ble=x-y ----variable----=--z--'.split())
        assert_equal(opts['variable'], ['x-y', '--z--'])
        assert_equal(args, [])

    def test_long_options_with_hyphens_only(self):
        args = '-----=value1'.split()
        assert_raises(DataError, self.ap.parse_args, args)

    def test_split_pythonpath(self):
        ap = ArgumentParser('ignored')
        data = [(['path'], ['path']),
                (['path1','path2'], ['path1','path2']),
                (['path1:path2'], ['path1','path2']),
                (['p1:p2:p3','p4','.'], ['p1','p2','p3','p4','.'])]
        if os.sep == '\\':
            data += [(['c:\\path'], ['c:\\path']),
                     (['c:\\path','d:\\path'], ['c:\\path','d:\\path']),
                     (['c:\\path:d:\\path'], ['c:\\path','d:\\path']),
                     (['c:/path:x:yy:d:\\path','c','.','x:/xxx'],
                      ['c:\\path', 'x', 'yy', 'd:\\path', 'c', '.', 'x:\\xxx'])]
        for inp, exp in data:
            assert_equal(ap._split_pythonpath(inp), exp)

    def test_get_pythonpath(self):
        ap = ArgumentParser('ignored')
        p1 = os.path.abspath('.')
        p2 = os.path.abspath('..')
        assert_equal(ap._get_pythonpath(p1), [p1])
        assert_equal(ap._get_pythonpath([p1,p2]), [p1,p2])
        assert_equal(ap._get_pythonpath([p1 + ':' + p2]), [p1,p2])
        assert_true(p1 in ap._get_pythonpath(os.path.join(p2,'*')))

    def test_arguments_are_globbed(self):
        _, args = self.ap.parse_args([__file__.replace('test_', '?????')])
        assert_equal(args, [__file__])
        # Needed to ensure that the globbed directory contains files
        globexpr = os.path.join(os.path.dirname(__file__), '*')
        _, args = self.ap.parse_args([globexpr])
        assert_true(len(args) > 1)

    def test_arguments_with_glob_patterns_arent_removed_if_they_dont_match(self):
        _, args = self.ap.parse_args(['*.non.existing', 'non.ex.??'])
        assert_equal(args, ['*.non.existing', 'non.ex.??'])

    def test_special_options_are_removed(self):
        ap = ArgumentParser('''Usage:
 -h --help
 -v --version
 --Argument-File path
 --option
''')
        opts, args = ap.parse_args(['--option'])
        assert_equal(opts, {'option': True})

    def test_special_options_can_be_turned_to_normal_options(self):
        ap = ArgumentParser('''Usage:
 -h --help
 -v --version
 --argumentfile path
''', auto_help=False, auto_version=False, auto_argumentfile=False)
        opts, args = ap.parse_args(['--help', '-v', '--arg', 'xxx'])
        assert_equal(opts, {'help': True, 'version': True, 'argumentfile': 'xxx'})

    def test_auto_pythonpath_is_deprecated(self):
        with warnings.catch_warnings(record=True) as w:
            ArgumentParser('-x', auto_pythonpath=False)
        assert_equal(str(w[0].message),
                     "ArgumentParser option 'auto_pythonpath' is deprecated "
                     "since Robot Framework 5.0.")

    def test_non_list_args(self):
        ap = ArgumentParser('''Options:
 -t --toggle
 -v --value value
 -m --multi multi *
''')
        opts, args = ap.parse_args(())
        assert_equal(opts, {'toggle': None,
                             'value': None,
                             'multi': []})
        assert_equal(args, [])
        opts, args = ap.parse_args(('-t', '-v', 'xxx', '-m', '1', '-m2', 'arg'))
        assert_equal(opts, {'toggle': True,
                             'value': 'xxx',
                             'multi': ['1', '2']})
        assert_equal(args, ['arg'])


class TestDefaultsFromEnvironmentVariables(unittest.TestCase):

    def setUp(self):
        os.environ['ROBOT_TEST_OPTIONS'] = '-t --value default -m1 --multi=2'
        self.ap = ArgumentParser('''Options:
 -t --toggle
 -v --value value
 -m --multi multi *
''', env_options='ROBOT_TEST_OPTIONS')

    def tearDown(self):
        os.environ.pop('ROBOT_TEST_OPTIONS')

    def test_flag(self):
        opts, args = self.ap.parse_args([])
        assert_equal(opts['toggle'], True)
        opts, args = self.ap.parse_args(['--toggle'])
        assert_equal(opts['toggle'], True)
        opts, args = self.ap.parse_args(['--notoggle'])
        assert_equal(opts['toggle'], False)

    def test_value(self):
        opts, args = self.ap.parse_args([])
        assert_equal(opts['value'], 'default')
        opts, args = self.ap.parse_args(['--value', 'given'])
        assert_equal(opts['value'], 'given')

    def test_multi_value(self):
        opts, args = self.ap.parse_args([])
        assert_equal(opts['multi'], ['1', '2'])
        opts, args = self.ap.parse_args(['-m3', '--multi', '4'])
        assert_equal(opts['multi'], ['1', '2', '3', '4'])

    def test_arguments(self):
        os.environ['ROBOT_TEST_OPTIONS'] = '-o opt arg1 arg2'
        ap = ArgumentParser('Usage:\n -o --opt value',
                            env_options='ROBOT_TEST_OPTIONS')
        opts, args = ap.parse_args([])
        assert_equal(opts['opt'], 'opt')
        assert_equal(args, ['arg1', 'arg2'])

    def test_environment_variable_not_set(self):
        ap = ArgumentParser('Usage:\n -o --opt value', env_options='NOT_SET')
        opts, args = ap.parse_args(['arg'])
        assert_equal(opts['opt'], None)
        assert_equal(args, ['arg'])


class TestArgumentValidation(unittest.TestCase):

    def test_check_args_with_correct_args(self):
        for arg_limits in [None, (1, 1), 1, (1,)]:
            ap = ArgumentParser(USAGE, arg_limits=arg_limits)
            assert_equal(ap.parse_args(['hello'])[1], ['hello'])

    def test_default_validation(self):
        ap = ArgumentParser(USAGE)
        for args in [(), ('1',), ('m', 'a', 'n', 'y')]:
            assert_equal(ap.parse_args(args)[1], list(args))

    def test_check_args_with_wrong_number_of_args(self):
        for limits in [1, (1, 1), (1, 2)]:
            ap = ArgumentParser('usage', arg_limits=limits)
            for args in [(), ('arg1', 'arg2', 'arg3')]:
                assert_raises(DataError, ap.parse_args, args)

    def test_check_variable_number_of_args(self):
        ap = ArgumentParser('usage:  robot.py [options] args', arg_limits=(1,))
        ap.parse_args(['one_is_ok'])
        ap.parse_args(['two', 'ok'])
        ap.parse_args(['this', 'should', 'also', 'work', '!'])
        assert_raises_with_msg(DataError, "Expected at least 1 argument, got 0.",
                               ap.parse_args, [])

    def test_argument_range(self):
        ap = ArgumentParser('usage:  test.py [options] args', arg_limits=(2,4))
        ap.parse_args(['1', '2'])
        ap.parse_args(['1', '2', '3', '4'])
        assert_raises_with_msg(DataError, "Expected 2 to 4 arguments, got 1.",
                               ap.parse_args, ['one is not enough'])

    def test_no_arguments(self):
        ap = ArgumentParser('usage:  test.py [options]', arg_limits=(0, 0))
        ap.parse_args([])
        assert_raises_with_msg(DataError, "Expected 0 arguments, got 2.",
                               ap.parse_args, ['1', '2'])

    def test_custom_validator_fails(self):
        def validate(options, args):
            raise AssertionError
        ap = ArgumentParser(USAGE2, validator=validate)
        assert_raises(AssertionError, ap.parse_args, [])

    def test_custom_validator_return_value(self):
        def validate(options, args):
            return options, [a.upper() for a in args]
        ap = ArgumentParser(USAGE2, validator=validate)
        opts, args = ap.parse_args(['-v', 'value', 'inp1', 'inp2'])
        assert_equal(opts['variable'], 'value')
        assert_equal(args, ['INP1', 'INP2'])


class TestPrintHelpAndVersion(unittest.TestCase):

    def setUp(self):
        self.ap = ArgumentParser(USAGE, version='1.0 alpha')
        self.ap2 = ArgumentParser(USAGE2)

    def test_print_help(self):
        assert_raises_with_msg(Information, USAGE2,
                               self.ap2.parse_args, ['--help'])

    def test_name_is_got_from_first_line_of_the_usage(self):
        assert_equal(self.ap.name, 'Example Tool')
        assert_equal(self.ap2.name, 'Just Name Here')

    def test_name_and_version_can_be_given(self):
        ap = ArgumentParser(USAGE, name='Kakkonen', version='2')
        assert_equal(ap.name, 'Kakkonen')
        assert_equal(ap.version, '2')

    def test_print_version(self):
        assert_raises_with_msg(Information, 'Example Tool 1.0 alpha',
                               self.ap.parse_args, ['--version'])

    def test_print_version_when_version_not_set(self):
        ap = ArgumentParser(' --version', name='Kekkonen')
        msg = assert_raises(Information, ap.parse_args, ['--version'])
        assert_equal(str(msg), 'Kekkonen %s' % get_full_version())

    def test_version_is_replaced_in_help(self):
        assert_raises_with_msg(Information, USAGE.replace('<VERSION>', '1.0 alpha'),
                               self.ap.parse_args, ['--help'])


if __name__ == "__main__":
    unittest.main()



import unittest
import os
from os.path import abspath

from robot.utils.asserts import assert_equal, assert_true
from robot.utils.text import (
    cut_long_message, get_console_length, getdoc, getshortdoc,
    pad_console_length, split_tags_from_doc, split_args_from_name_or_path,
    _count_line_lengths, MAX_ERROR_LINES, _MAX_ERROR_LINE_LENGTH,
    _ERROR_CUT_EXPLN
)


_HALF_ERROR_LINES = MAX_ERROR_LINES // 2


class NoCutting(unittest.TestCase):

    def test_empty_string(self):
        self._assert_no_cutting('')

    def test_short_message(self):
        self._assert_no_cutting('bar')

    def test_few_short_lines(self):
        self._assert_no_cutting('foo\nbar\nzap\nphello World!')

    def test_max_number_of_short_lines(self):
        self._assert_no_cutting('short line\n' * MAX_ERROR_LINES)

    def _assert_no_cutting(self, msg):
        assert_equal(cut_long_message(msg), msg)


class TestCutting(unittest.TestCase):

    def setUp(self):
        self.lines = ['my error message %d' % i for i in range(MAX_ERROR_LINES+1)]
        self.result = cut_long_message('\n'.join(self.lines)).splitlines()
        self.limit = _HALF_ERROR_LINES

    def test_more_than_max_number_of_lines(self):
        assert_equal(len(self.result), MAX_ERROR_LINES+1)

    def test_cut_message_is_present(self):
        assert_true(_ERROR_CUT_EXPLN in self.result)

    def test_cut_message_starts_with_original_lines(self):
        expected = self.lines[:self.limit]
        actual = self.result[:self.limit]
        assert_equal(actual, expected)

    def test_cut_message_ends_with_original_lines(self):
        expected = self.lines[-self.limit:]
        actual = self.result[-self.limit:]
        assert_equal(actual, expected)


class TestCuttingWithLinesLongerThanMax(unittest.TestCase):

    def setUp(self):
        self.lines = ['line %d' % i for i in range(MAX_ERROR_LINES-1)]
        self.lines.append('x' * (_MAX_ERROR_LINE_LENGTH+1))
        self.result = cut_long_message('\n'.join(self.lines)).splitlines()

    def test_cut_message_present(self):
        assert_true(_ERROR_CUT_EXPLN in self.result)

    def test_correct_number_of_lines(self):
        assert_equal(sum(_count_line_lengths(self.result)), MAX_ERROR_LINES+1)

    def test_correct_lines(self):
        expected = self.lines[:_HALF_ERROR_LINES] + [_ERROR_CUT_EXPLN] \
                + self.lines[-_HALF_ERROR_LINES+1:]
        assert_equal(self.result, expected)

    def test_every_line_longer_than_limit(self):
        # sanity check
        lines = [('line %d' % i) * _MAX_ERROR_LINE_LENGTH for i in range(MAX_ERROR_LINES+2)]
        result = cut_long_message('\n'.join(lines)).splitlines()
        assert_true(_ERROR_CUT_EXPLN in result)
        assert_equal(result[0], lines[0])
        assert_equal(result[-1], lines[-1])
        assert_true(sum(_count_line_lengths(result)) <= MAX_ERROR_LINES+1)


class TestCutHappensInsideLine(unittest.TestCase):

    def test_long_line_cut_before_cut_message(self):
        lines = ['line %d' % i for i in range(MAX_ERROR_LINES)]
        index = _HALF_ERROR_LINES - 1
        lines[index] = 'abcdefgh' * _MAX_ERROR_LINE_LENGTH
        result = cut_long_message('\n'.join(lines)).splitlines()
        self._assert_basics(result, lines)
        expected = lines[index][:_MAX_ERROR_LINE_LENGTH-3] + '...'
        assert_equal(result[index], expected)

    def test_long_line_cut_after_cut_message(self):
        lines = ['line %d' % i for i in range(MAX_ERROR_LINES)]
        index = _HALF_ERROR_LINES
        lines[index] = 'abcdefgh' * _MAX_ERROR_LINE_LENGTH
        result = cut_long_message('\n'.join(lines)).splitlines()
        self._assert_basics(result, lines)
        expected = '...' + lines[index][-_MAX_ERROR_LINE_LENGTH+3:]
        assert_equal(result[index+1], expected)

    def test_one_huge_line(self):
        result = cut_long_message('0123456789' * MAX_ERROR_LINES * _MAX_ERROR_LINE_LENGTH)
        self._assert_basics(result.splitlines())
        assert_true(result.startswith('0123456789'))
        assert_true(result.endswith('0123456789'))
        assert_true('...\n'+_ERROR_CUT_EXPLN+'\n...' in result)

    def _assert_basics(self, result, input=None):
        assert_equal(sum(_count_line_lengths(result)), MAX_ERROR_LINES+1)
        assert_true(_ERROR_CUT_EXPLN in result)
        if input:
            assert_equal(result[0], input[0])
            assert_equal(result[-1], input[-1])


class TestCountLines(unittest.TestCase):

    def test_no_lines(self):
        assert_equal(_count_line_lengths([]), [])

    def test_empty_line(self):
        assert_equal(_count_line_lengths(['']), [1])

    def test_shorter_than_max_lines(self):
        lines = ['', '1', 'foo', 'barz and fooz', '', 'a bit longer line', '',
                 'This is a somewhat longer (but not long enough) error message']
        assert_equal(_count_line_lengths(lines), [1] * len(lines))

    def test_longer_than_max_lines(self):
        lines = [ '1' * i * (_MAX_ERROR_LINE_LENGTH+3) for i in range(4) ]
        assert_equal(_count_line_lengths(lines), [1,2,3,4])

    def test_boundary(self):
        b = _MAX_ERROR_LINE_LENGTH
        lengths = [b-1, b, b+1, 2*b-1, 2*b, 2*b+1, 7*b-1, 7*b, 7*b+1]
        lines = [ 'e'*length for length in lengths ]
        assert_equal(_count_line_lengths(lines), [1, 1, 2, 2, 2, 3, 7, 7, 8])


class TestConsoleWidth(unittest.TestCase):
    len16_asian = u'\u6c49\u5b57\u5e94\u8be5\u6b63\u786e\u5bf9\u9f50'
    ten_normal = u'1234567890'
    mixed_26 = u'012345\u6c49\u5b57\u5e94\u8be5\u6b63\u786e\u5bf9\u9f567890'
    nfd = u'A\u030Abo'

    def test_console_width(self):
        assert_equal(get_console_length(self.ten_normal), 10)

    def test_east_asian_width(self):
        assert_equal(get_console_length(self.len16_asian), 16)

    def test_combining_width(self):
        assert_equal(get_console_length(self.nfd), 3)

    def test_cut_right(self):
        assert_equal(pad_console_length(self.ten_normal, 5), '12...')
        assert_equal(pad_console_length(self.ten_normal, 15), self.ten_normal+' '*5)
        assert_equal(pad_console_length(self.ten_normal, 10), self.ten_normal)

    def test_cut_east_asian(self):
        assert_equal(pad_console_length(self.len16_asian, 10), u'\u6c49\u5b57\u5e94... ')
        assert_equal(pad_console_length(self.mixed_26, 11), u'012345\u6c49...')


class TestDocSplitter(unittest.TestCase):

    def test_doc_without_tags(self):
        docs = ["Single doc line.",
                """Hello, we dont have tags here.

                No sir. No tags.""",
                "Now Tags: must, start from beginning of the row",
                "   We strip  the trailing whitespace  \n \n"]
        for doc in docs:
            self._assert_doc_and_tags(doc, doc.rstrip(), [])

    def _assert_doc_and_tags(self, original, expected_doc, expected_tags):
        doc, tags = split_tags_from_doc(original)
        assert_equal(doc, expected_doc)
        assert_equal(tags, expected_tags)

    def test_doc_with_tags(self):
        sets = [
            ('Tags: foo, bar',                  '',             ['foo', 'bar']),
            ('  Tags: foo   ',                  '',             ['foo']),
            ('Hello\nTags: foo, bar',           'Hello',        ['foo', 'bar']),
            ('Tags: bar\n   Tags: foo   ',      'Tags: bar',    ['foo']),
            ('Tags: bar, Tags:, foo   ',        '',             ['bar', 'Tags:', 'foo']),
            ('tags: foo',                       '',             ['foo']),
            ('   tags: foo ,  bar  ',           '',             ['foo', 'bar']),
            ('Hello\n   taGS: foo, bar',        'Hello',        ['foo', 'bar']),
            (' Hello\n   taGS: f, b \n\n \n',   ' Hello',       ['f', 'b']),
            ('Hello\nNl  \n  \nTags: foo',      'Hello\nNl',    ['foo']),
        ]
        for original, exp_doc, exp_tags in sets:
            self._assert_doc_and_tags(original, exp_doc, exp_tags)
            self._assert_doc_and_tags(original+'\n', exp_doc, exp_tags)


class TestSplitArgsFromNameOrPath(unittest.TestCase):

    def setUp(self):
        self.method = split_args_from_name_or_path

    def test_with_no_args(self):
        assert not os.path.exists('name'), 'does not work if you have name folder!'
        assert_equal(self.method('name'), ('name', []))

    def test_with_args(self):
        assert not os.path.exists('name'), 'does not work if you have name folder!'
        assert_equal(self.method('name:arg'), ('name', ['arg']))
        assert_equal(self.method('listener:v1:v2:v3'), ('listener', ['v1', 'v2', 'v3']))
        assert_equal(self.method('aa:bb:cc'), ('aa', ['bb', 'cc']))

    def test_empty_args(self):
        assert not os.path.exists('foo'), 'does not work if you have foo folder!'
        assert_equal(self.method('foo:'), ('foo', ['']))
        assert_equal(self.method('bar:arg1::arg3'), ('bar', ['arg1', '', 'arg3']))
        assert_equal(self.method('3:'), ('3', ['']))

    def test_semicolon_as_separator(self):
        assert_equal(self.method('name;arg'), ('name', ['arg']))
        assert_equal(self.method('name;1;2;3'), ('name', ['1', '2', '3']))
        assert_equal(self.method('name;'), ('name', ['']))

    def test_alternative_separator_in_value(self):
        assert_equal(self.method('name;v:1;v:2'), ('name', ['v:1', 'v:2']))
        assert_equal(self.method('name:v;1:v;2'), ('name', ['v;1', 'v;2']))

    def test_windows_path_without_args(self):
        assert_equal(self.method('C:\\name.py'), ('C:\\name.py', []))
        assert_equal(self.method('X:\\APPS\\listener'), ('X:\\APPS\\listener', []))
        assert_equal(self.method('C:/varz.py'), ('C:/varz.py', []))

    def test_windows_path_with_args(self):
        assert_equal(self.method('C:\\name.py:arg1'), ('C:\\name.py', ['arg1']))
        assert_equal(self.method('D:\\APPS\\listener:v1:b2:z3'),
                      ('D:\\APPS\\listener', ['v1', 'b2', 'z3']))
        assert_equal(self.method('C:/varz.py:arg'), ('C:/varz.py', ['arg']))
        assert_equal(self.method('C:\\file.py:arg;with;alternative;separator'),
                      ('C:\\file.py', ['arg;with;alternative;separator']))

    def test_windows_path_with_semicolon_separator(self):
        assert_equal(self.method('C:\\name.py;arg1'), ('C:\\name.py', ['arg1']))
        assert_equal(self.method('D:\\APPS\\listener;v1;b2;z3'),
                      ('D:\\APPS\\listener', ['v1', 'b2', 'z3']))
        assert_equal(self.method('C:/varz.py;arg'), ('C:/varz.py', ['arg']))
        assert_equal(self.method('C:\\file.py;arg:with:alternative:separator'),
                      ('C:\\file.py', ['arg:with:alternative:separator']))

    def test_existing_paths_are_made_absolute(self):
        path = 'robot-framework-unit-test-file-12q3405909qasf'
        open(path, 'w').close()
        try:
            assert_equal(self.method(path), (abspath(path), []))
            assert_equal(self.method(path+':arg'), (abspath(path), ['arg']))
        finally:
            os.remove(path)

    def test_existing_path_with_colons(self):
        # Colons aren't allowed in Windows paths (other than in "c:")
        if os.sep == '\\':
            return
        path = 'robot:framework:test:1:2:42'
        os.mkdir(path)
        try:
            assert_equal(self.method(path), (abspath(path), []))
        finally:
            os.rmdir(path)


class TestGetdoc(unittest.TestCase):

    def test_no_doc(self):
        def func():
            pass
        assert_equal(getdoc(func), '')

    def test_one_line_doc(self):
        def func():
            """My documentation."""
        assert_equal(getdoc(func), 'My documentation.')

    def test_multiline_doc(self):
        class Class:
            """My doc.

            In multiple lines.
            """
        assert_equal(getdoc(Class), 'My doc.\n\nIn multiple lines.')
        assert_equal(getdoc(Class), getdoc(Class()))

    def test_unicode_doc(self):
        class Class:
            def meth(self):
                u"""Hyv\xe4 \xe4iti!"""
        assert_equal(getdoc(Class.meth), u'Hyv\xe4 \xe4iti!')
        assert_equal(getdoc(Class.meth), getdoc(Class().meth))


class TestGetshortdoc(unittest.TestCase):

    def test_empty(self):
        self._verify('', '')

    def test_one_line(self):
        self._verify('Hello, world!', 'Hello, world!')

    def test_multiline_with_one_line_short_doc(self):
        self._verify('''\
This is the short doc. Nicely in one line.

This is the remainder of the doc.
''', 'This is the short doc. Nicely in one line.')

    def test_only_short_doc_split_to_many_lines(self):
        self._verify('This time short doc is\nsplit to multiple lines.',
                     'This time short doc is\nsplit to multiple lines.')

    def test_multiline_with_multiline_short_doc(self):
        self._verify('''\
This is the short doc.
Nicely in multiple
lines.

This is the remainder of the doc.
''', 'This is the short doc.\nNicely in multiple\nlines.')

    def test_line_with_only_spaces_is_considered_empty(self):
        self._verify('Short\ndoc\n\n    \nignored', 'Short\ndoc')

    def test_doc_from_object(self):
        def func():
            """This is short doc
            in multiple lines.

            This is the remainder.
            """
        self._verify(func, 'This is short doc\nin multiple lines.')

    def _verify(self, doc, expected):
        assert_equal(getshortdoc(doc), expected)
        assert_equal(getshortdoc(doc, linesep=' '), expected.replace('\n', ' '))


if __name__ == '__main__':
    unittest.main()



from robot.api.deco import keyword


class InvalidKeywords:

    @keyword('Invalid embedded ${args}')
    def invalid_embedded(self):
        pass

    def duplicate_name(self):
        pass

    def duplicateName(self):
        pass

    @keyword('Same ${embedded}')
    def dupe_with_embedded_1(self, arg):
        pass

    @keyword('same ${match}')
    def dupe_with_embedded_2(self, arg):
        """This is an error only at run time."""
        pass



import sys


class Listener:
    ROBOT_LISTENER_API_VERSION = 2

    def __init__(self, name='X'):
        self.name = name

    def start_suite(self, name, attrs):
        self._log("from listener {0}".format(self.name))

    def close(self):
        self._log("listener close")

    def report_file(self, path):
        self._log("report {0}".format(path))

    def log_file(self, path):
        self._log("log {0}".format(path))

    def output_file(self, path):
        self._log("output {0}".format(path))

    def _log(self, message):
        sys.__stdout__.write("[{0}]\n".format(message))



import sys
from robot.api import logger


print('*WARN* Warning via stdout in import')
print('Info via stderr in import', file=sys.stderr)
logger.warn('Warning via API in import')


def keyword():
    pass



import unittest
from pathlib import Path


class TestZipSafe(unittest.TestCase):

    def test_no_unsafe__file__usages(self):
        root = Path(__file__).absolute().parent.parent.parent / 'src/robot'

        def unsafe__file__usage(line, path):
            if ('__file__' not in line or '# zipsafe' in line
                    or path.parent == root / 'htmldata/testdata'):
                return False
            return '__file__' in line.replace("'__file__'", '').replace('"__file__"', '')

        for path in root.rglob('*.py'):
            with path.open(encoding='UTF-8') as file:
                for lineno, line in enumerate(file, start=1):
                    if unsafe__file__usage(line, path):
                        raise AssertionError(f'Unsafe __file__ usage in {path} '
                                             f'on line {lineno}.')


if __name__ == '__main__':
    unittest.main()



import unittest

from robot.utils.asserts import assert_equal

from robot.utils.markuputils import html_escape, html_format, attribute_escape
from robot.utils.htmlformatters import TableFormatter

_format_table = TableFormatter()._format_table


def assert_escape_and_format(inp, exp_escape=None, exp_format=None):
    if exp_escape is None:
        exp_escape = str(inp)
    if exp_format is None:
        exp_format = exp_escape
    exp_format = '<p>%s</p>' % exp_format.replace('\n', ' ')
    escape = html_escape(inp)
    format = html_format(inp)
    assert_equal(escape, exp_escape,
                 'ESCAPE:\n%r   =!\n%r' % (escape, exp_escape), values=False)
    assert_equal(format, exp_format,
                 'FORMAT:\n%r   =!\n%r' % (format, exp_format), values=False)


def assert_format(inp, exp=None, p=False):
    exp = exp if exp is not None else inp
    if p:
        exp = '<p>%s</p>' % exp
    assert_equal(html_format(inp), exp)


def assert_escape(inp, exp=None):
    exp = exp if exp is not None else inp
    assert_equal(html_escape(inp), exp)


class TestHtmlEscape(unittest.TestCase):

    def test_no_changes(self):
        for inp in ['', 'nothing to change']:
            assert_escape(inp)

    def test_newlines_and_paragraphs(self):
        for inp in ['Text on first line.\nText on second line.',
                    '1 line\n2 line\n3 line\n4 line\n5 line\n',
                    'Para 1 line 1\nP1 L2\n\nP2 L1\nP2 L1\n\nP3 L1\nP3 L2',
                     'Multiple empty lines\n\n\n\n\nbetween these lines']:
            assert_escape(inp)


class TestEntities(unittest.TestCase):

    def test_entities(self):
        for char, entity in [('<','&lt;'), ('>','&gt;'), ('&','&amp;')]:
            for inp, exp in [(char, entity),
                             ('text %s' % char, 'text %s' % entity),
                             ('-%s-%s-' % (char, char),
                              '-%s-%s-' % (entity, entity)),
                             ('"%s&%s"' % (char, char),
                              '"%s&amp;%s"' % (entity, entity))]:
                assert_escape_and_format(inp, exp)


class TestUrlsToLinks(unittest.TestCase):

    def test_not_urls(self):
        for no_url in ['http no link', 'http:/no', '123://no',
                       '1a://no', 'http://', 'http:// no']:
            assert_escape_and_format(no_url)

    def test_simple_urls(self):
        for link in ['http://robot.fi', 'https://r.fi/', 'FTP://x.y.z/p/f.txt',
                     'a23456://link', 'file:///c:/temp/xxx.yyy']:
            exp = '<a href="%s">%s</a>' % (link, link)
            assert_escape_and_format(link, exp)
            for end in [',', '.', ';', ':', '!', '?', '...', '!?!', ' hello' ]:
                assert_escape_and_format(link+end, exp+end)
                assert_escape_and_format('xxx '+link+end, 'xxx '+exp+end)
            for start, end in [('(',')'), ('[',']'), ('"','"'), ("'","'")]:
                assert_escape_and_format(start+link+end, start+exp+end)

    def test_complex_urls_and_surrounding_content(self):
        for inp, exp in [
                ('hello http://link world',
                 'hello <a href="http://link">http://link</a> world'),
                ('multi\nhttp://link\nline',
                 'multi\n<a href="http://link">http://link</a>\nline'),
                ('http://link, ftp://link2.',
                 '<a href="http://link">http://link</a>, '
                 '<a href="ftp://link2">ftp://link2</a>.'),
                ('x (git+ssh://yy, z)',
                 'x (<a href="git+ssh://yy">git+ssh://yy</a>, z)'),
                ('(http://x.com/blah_(wikipedia)#cite-1)',
                 '(<a href="http://x.com/blah_(wikipedia)#cite-1">http://x.com/blah_(wikipedia)#cite-1</a>)'),
                ('x-yojimbo-item://6303,E4C1,6A6E, FOO',
                 '<a href="x-yojimbo-item://6303,E4C1,6A6E">x-yojimbo-item://6303,E4C1,6A6E</a>, FOO'),
                ('Hello http://one, ftp://kaksi/; "gopher://3.0"',
                 'Hello <a href="http://one">http://one</a>, '
                 '<a href="ftp://kaksi/">ftp://kaksi/</a>; '
                 '"<a href="gopher://3.0">gopher://3.0</a>"'),
                ("'{https://issues/3231}'",
                 "'{<a href=\"https://issues/3231\">https://issues/3231</a>}'")]:
            assert_escape_and_format(inp, exp)

    def test_image_urls(self):
        link = '(<a href="%s">%s</a>)'
        img = '(<img src="%s" title="%s">)'
        for ext in ['jpg', 'jpeg', 'png', 'gif', 'bmp', 'svg']:
            url = 'foo://bar/zap.%s' % ext
            uprl = url.upper()
            inp = '(%s)' % url
            assert_escape_and_format(inp, link % (url, url), img % (url, url))
            assert_escape_and_format(inp.upper(), link % (uprl, uprl),
                                    img % (uprl, uprl))

    def test_url_with_chars_needing_escaping(self):
        for items in [
            ('http://foo"bar',
             '<a href="http://foo&quot;bar">http://foo"bar</a>'),
            ('ftp://<&>/',
             '<a href="ftp://&lt;&amp;&gt;/">ftp://&lt;&amp;&gt;/</a>'),
            ('http://x&".png',
            '<a href="http://x&amp;&quot;.png">http://x&amp;".png</a>',
            '<img src="http://x&amp;&quot;.png" title="http://x&amp;&quot;.png">')
        ]:
            assert_escape_and_format(*items)


class TestFormatParagraph(unittest.TestCase):

    def test_empty(self):
        assert_format('', '')

    def test_single_line(self):
        assert_format('foo', '<p>foo</p>')

    def test_multi_line(self):
        assert_format('foo\nbar', '<p>foo bar</p>')

    def test_leading_and_trailing_spaces(self):
        assert_format('  foo  \n  bar', '<p>foo bar</p>')

    def test_multiple_paragraphs(self):
        assert_format('P\n1\n\nP 2', '<p>P 1</p>\n<p>P 2</p>')

    def test_leading_empty_line(self):
        assert_format('\nP', '<p>P</p>')

    def test_other_formatted_content_before_paragraph(self):
        assert_format('---\nP', '<hr>\n<p>P</p>')
        assert_format('| PRE \nP', '<pre>\nPRE\n</pre>\n<p>P</p>')

    def test_other_formatted_content_after_paragraph(self):
        assert_format('P\n---', '<p>P</p>\n<hr>')
        assert_format('P\n| PRE \n', '<p>P</p>\n<pre>\nPRE\n</pre>')


class TestHtmlFormatInlineStyles(unittest.TestCase):

    def test_bold_once(self):
        for inp, exp in [('*bold*', '<b>bold</b>'),
                         ('*b*', '<b>b</b>'),
                         ('*many bold words*', '<b>many bold words</b>'),
                         (' *bold*', '<b>bold</b>'),
                         ('*bold* ', '<b>bold</b>'),
                         ('xx *bold*', 'xx <b>bold</b>'),
                         ('*bold* xx', '<b>bold</b> xx'),
                         ('***', '<b>*</b>'),
                         ('****', '<b>**</b>'),
                         ('*****', '<b>***</b>')]:
            assert_format(inp, exp, p=True)

    def test_bold_multiple_times(self):
        for inp, exp in [('*bold* *b* not bold *b3* not',
                          '<b>bold</b> <b>b</b> not bold <b>b3</b> not'),
                         ('not b *this is b* *more b words here*',
                          'not b <b>this is b</b> <b>more b words here</b>'),
                         ('*** not *b* ***',
                          '<b>*</b> not <b>b</b> <b>*</b>')]:
            assert_format(inp, exp, p=True)

    def test_bold_on_multiple_lines(self):
        inp = 'this is *bold*\nand *this*\nand *that*'
        exp = 'this is <b>bold</b> and <b>this</b> and <b>that</b>'
        assert_format(inp, exp, p=True)
        assert_format('this *works\ntoo!*', 'this <b>works too!</b>', p=True)

    def test_not_bolded_if_no_content(self):
        assert_format('**', p=True)

    def test_asterisk_in_the_middle_of_word_is_ignored(self):
        for inp, exp in [('aa*notbold*bbb', None),
                         ('*bold*still bold*', '<b>bold*still bold</b>'),
                         ('a*not*b c*still not*d', None),
                         ('*b*b2* -*n*- *b3*', '<b>b*b2</b> -*n*- <b>b3</b>')]:
            assert_format(inp, exp, p=True)

    def test_asterisk_alone_does_not_start_bolding(self):
        for inp, exp in [('*', None),
                         (' * ', '*'),
                         ('* not *', None),
                         (' * not * ', '* not *'),
                         ('* not*', None),
                         ('*bold *', '<b>bold </b>'),
                         ('* *b* *', '* <b>b</b> *'),
                         ('*bold * not*', '<b>bold </b> not*'),
                         ('*bold * not*not* *b*',
                          '<b>bold </b> not*not* <b>b</b>')]:
            assert_format(inp, exp, p=True)

    def test_italic_once(self):
        for inp, exp in [('_italic_', '<i>italic</i>'),
                         ('_i_', '<i>i</i>'),
                         ('_many italic words_', '<i>many italic words</i>'),
                         (' _italic_', '<i>italic</i>'),
                         ('_italic_ ', '<i>italic</i>'),
                         ('xx _italic_', 'xx <i>italic</i>'),
                         ('_italic_ xx', '<i>italic</i> xx')]:
            assert_format(inp, exp, p=True)

    def test_italic_multiple_times(self):
        for inp, exp in [('_italic_ _i_ not italic _i3_ not',
                          '<i>italic</i> <i>i</i> not italic <i>i3</i> not'),
                         ('not i _this is i_ _more i words here_',
                          'not i <i>this is i</i> <i>more i words here</i>')]:
            assert_format(inp, exp, p=True)

    def test_not_italiced_if_no_content(self):
        assert_format('__', p=True)

    def test_not_italiced_many_underlines(self):
        for inp in ['___', '____', '_________', '__len__']:
            assert_format(inp, p=True)

    def test_underscore_in_the_middle_of_word_is_ignored(self):
        for inp, exp in [('aa_notitalic_bbb', None),
                         ('_ital_still ital_', '<i>ital_still ital</i>'),
                         ('a_not_b c_still not_d', None),
                         ('_i_i2_ -_n_- _i3_', '<i>i_i2</i> -_n_- <i>i3</i>')]:
            assert_format(inp, exp, p=True)

    def test_underscore_alone_does_not_start_italicing(self):
        for inp, exp in [('_', None),
                         (' _ ', '_'),
                         ('_ not _', None),
                         (' _ not _ ', '_ not _'),
                         ('_ not_', None),
                         ('_italic _', '<i>italic </i>'),
                         ('_ _i_ _', '_ <i>i</i> _'),
                         ('_italic _ not_', '<i>italic </i> not_'),
                         ('_italic _ not_not_ _i_',
                          '<i>italic </i> not_not_ <i>i</i>')]:
            assert_format(inp, exp, p=True)

    def test_bold_and_italic(self):
        for inp, exp in [('*b* _i_', '<b>b</b> <i>i</i>')]:
            assert_format(inp, exp, p=True)

    def test_bold_and_italic_works_with_punctuation_marks(self):
        for bef, aft in [('(',''), ('"',''), ("'",''), ('(\'"(',''),
                         ('',')'), ('','"'), ('',','), ('','"\').,!?!?:;'),
                         ('(',')'), ('"','"'), ('("\'','\'";)'), ('"','..."')]:
            for inp, exp in [('*bold*','<b>bold</b>'),
                             ('_ital_','<i>ital</i>'),
                             ('*b* _i_','<b>b</b> <i>i</i>')]:
                assert_format(bef + inp + aft, bef + exp + aft, p=True)

    def test_bold_italic(self):
        for inp, exp in [('_*bi*_', '<i><b>bi</b></i>'),
                         ('_*bold ital*_', '<i><b>bold ital</b></i>'),
                         ('_*bi* i_', '<i><b>bi</b> i</i>'),
                         ('_*bi_ b*', '<i><b>bi</i> b</b>'),
                         ('_i *bi*_', '<i>i <b>bi</b></i>'),
                         ('*b _bi*_', '<b>b <i>bi</b></i>')]:
            assert_format(inp, exp, p=True)

    def test_code_once(self):
        for inp, exp in [('``code``', '<code>code</code>'),
                         ('``c``', '<code>c</code>'),
                         ('``many code words``', '<code>many code words</code>'),
                         (' ``leading space``', '<code>leading space</code>'),
                         ('``trailing space`` ', '<code>trailing space</code>'),
                         ('xx ``code``', 'xx <code>code</code>'),
                         ('``code`` xx', '<code>code</code> xx')]:
            assert_format(inp, exp, p=True)

    def test_code_multiple_times(self):
        for inp, exp in [('``code`` ``c`` not ``c3`` not',
                          '<code>code</code> <code>c</code> not <code>c3</code> not'),
                         ('not c ``this is c`` ``more c words here``',
                          'not c <code>this is c</code> <code>more c words here</code>')]:
            assert_format(inp, exp, p=True)

    def test_not_coded_if_no_content(self):
        assert_format('````', p=True)

    def test_not_codeed_many_underlines(self):
        for inp in ['``````', '````````', '``````````````````', '````len````']:
            assert_format(inp, p=True)

    def test_backtics_in_the_middle_of_word_are_ignored(self):
        for inp, exp in [('aa``notcode``bbb', None),
                         ('``code``still code``', '<code>code``still code</code>'),
                         ('a``not``b c``still not``d', None),
                         ('``c``c2`` -``n``- ``c3``', '<code>c``c2</code> -``n``- <code>c3</code>')]:
            assert_format(inp, exp, p=True)

    def test_backtics_alone_do_not_start_codeing(self):
        for inp, exp in [('``', None),
                         (' `` ', '``'),
                         ('`` not ``', None),
                         (' `` not `` ', '`` not ``'),
                         ('`` not``', None),
                         ('``code ``', '<code>code </code>'),
                         ('`` ``b`` ``', '`` <code>b</code> ``'),
                         ('``code `` not``', '<code>code </code> not``'),
                         ('``code `` not``not`` ``c``',
                          '<code>code </code> not``not`` <code>c</code>')]:
            assert_format(inp, exp, p=True)


class TestHtmlFormatCustomLinks(unittest.TestCase):
    image_extensions = ('jpg', 'jpeg', 'PNG', 'Gif', 'bMp', 'svg')

    def test_text_with_text(self):
        assert_format('[link.html|title]', '<a href="link.html">title</a>', p=True)
        assert_format('[link|t|i|t|l|e]', '<a href="link">t|i|t|l|e</a>', p=True)

    def test_text_with_image(self):
        for ext in self.image_extensions:
            assert_format(
                '[link|img.%s]' % ext,
                '<a href="link"><img src="img.%s" title="link"></a>' % ext,
                p=True
            )

    def test_image_with_text(self):
        for ext in self.image_extensions:
            img = 'doc/images/robot.%s' % ext
            assert_format(
                'Robot [%s|robot]!' % img,
                'Robot <img src="%s" title="robot">!' % img,
                p=True
            )
            assert_format(
                'Robot [%s|]!' % img,
                'Robot <img src="%s" title="%s">!' % (img, img),
                p=True
            )

    def test_image_with_image(self):
        for ext in self.image_extensions:
            assert_format(
                '[X.%s|Y.%s]' % (ext, ext),
                '<a href="X.%s"><img src="Y.%s" title="X.%s"></a>' % ((ext,)*3),
                p=True
            )

    def test_text_with_data_uri_image(self):
        uri = 'data:image/png;base64,oooxxx='
        assert_format(
            '[robot.html|%s]' % uri,
            '<a href="robot.html"><img src="%s" title="robot.html"></a>' % uri,
            p=True
        )

    def test_data_uri_image_with_text(self):
        uri = 'data:image/png;base64,oooxxx='
        assert_format(
            '[%s|Robot rocks!]' % uri,
            '<img src="%s" title="Robot rocks!">' % uri,
            p=True
        )

    def test_image_with_data_uri_image(self):
        uri = 'data:image/png;base64,oooxxx='
        assert_format(
            '[image.jpg|%s]' % uri,
            '<a href="image.jpg"><img src="%s" title="image.jpg"></a>' % uri,
            p=True
        )

    def test_data_uri_image_with_data_uri_image(self):
        uri = 'data:image/png;base64,oooxxx='
        assert_format(
            '[%s|%s]' % (uri, uri),
            '<a href="%s"><img src="%s" title="%s"></a>' % (uri, uri, uri),
            p=True
        )

    def test_link_is_required(self):
        assert_format('[|]', '[|]', p=True)

    def test_spaces_are_stripped(self):
        assert_format('[ link.html  | title words  ]',
                      '<a href="link.html">title words</a>', p=True)

    def test_newlines_inside_text(self):
        assert_format('[http://url|text\non\nmany\nlines]',
                      '<a href="http://url">text on many lines</a>', p=True)

    def test_newline_after_pipe(self):
        assert_format('[http://url|\nwrapping was needed]',
                      '<a href="http://url">wrapping was needed</a>', p=True)

    def test_url_and_link(self):
        assert_format('http://url [link|title]',
                      '<a href="http://url">http://url</a> <a href="link">title</a>',
                      p=True)

    def test_link_as_url(self):
        assert_format('[http://url|title]', '<a href="http://url">title</a>', p=True)

    def test_multiple_links(self):
        assert_format('start [link|img.png] middle [link.html|title] end',
                      'start <a href="link"><img src="img.png" title="link"></a> '
                      'middle <a href="link.html">title</a> end', p=True)

    def test_multiple_links_and_urls(self):
        assert_format('[L|T]ftp://url[X|Y][http://u2]',
                      '<a href="L">T</a><a href="ftp://url">ftp://url</a>'
                      '<a href="X">Y</a>[<a href="http://u2">http://u2</a>]', p=True)

    def test_escaping(self):
        assert_format('["|<&>]', '<a href="&quot;">&lt;&amp;&gt;</a>', p=True)
        assert_format('[<".jpg|">]', '<img src="&lt;&quot;.jpg" title="&quot;&gt;">', p=True)

    def test_formatted_link(self):
        assert_format('*[link.html|title]*', '<b><a href="link.html">title</a></b>', p=True)

    def test_link_in_table(self):
        assert_format('| [link.html|title] |', '''\
<table border="1">
<tr>
<td><a href="link.html">title</a></td>
</tr>
</table>''')


class TestHtmlFormatTable(unittest.TestCase):

    def test_one_row_table(self):
        inp = '| one | two |'
        exp = _format_table([['one','two']])
        assert_format(inp, exp)

    def test_multi_row_table(self):
        inp = '| 1.1 | 1.2 | 1.3 |\n| 2.1 | 2.2 |\n| 3.1 | 3.2 | 3.3 |\n'
        exp = _format_table([['1.1','1.2','1.3'],
                             ['2.1','2.2'],
                             ['3.1','3.2','3.3']])
        assert_format(inp, exp)

    def test_table_with_extra_spaces(self):
        inp = '  |   1.1   |  1.2   |  \n   | 2.1 | 2.2 |    '
        exp = _format_table([['1.1','1.2',],['2.1','2.2']])
        assert_format(inp, exp)

    def test_table_with_one_space_empty_cells(self):
        inp = '''
| 1.1 | 1.2 | |
| 2.1 | | 2.3 |
| | 3.2 | 3.3 |
| 4.1 | | |
| | 5.2 | |
| | | 6.3 |
| | | |
'''[1:-1]
        exp = _format_table([['1.1','1.2',''],
                             ['2.1','','2.3'],
                             ['','3.2','3.3'],
                             ['4.1','',''],
                             ['','5.2',''],
                             ['','','6.3'],
                             ['','','']])
        assert_format(inp, exp)

    def test_one_column_table(self):
        inp = '|  one column |\n| |\n  |  |  \n| 2 | col |\n|          |'
        exp = _format_table([['one column'],[''],[''],['2','col'],['']])
        assert_format(inp, exp)

    def test_table_with_other_content_around(self):
        inp = '''before table
| in | table |
| still | in |

 after table
'''
        exp = '<p>before table</p>\n' \
            + _format_table([['in','table'],['still','in']]) \
            + '\n<p>after table</p>'
        assert_format(inp, exp)

    def test_multiple_tables(self):
        inp = '''before tables
| table | 1 |
| still | 1 |

between

| table | 2 |
between
| 3.1.1 | 3.1.2 | 3.1.3 |
| 3.2.1 | 3.2.2 | 3.2.3 |
| 3.3.1 | 3.3.2 | 3.3.3 |

| t | 4 |
|   |   |

after
'''
        exp = '<p>before tables</p>\n' \
            + _format_table([['table','1'],['still','1']]) \
            + '\n<p>between</p>\n' \
            + _format_table([['table','2']]) \
            + '\n<p>between</p>\n' \
            + _format_table([['3.1.1','3.1.2','3.1.3'],
                             ['3.2.1','3.2.2','3.2.3'],
                             ['3.3.1','3.3.2','3.3.3']]) \
            + '\n' \
            + _format_table([['t','4'],['','']]) \
            + '\n<p>after</p>'
        assert_format(inp, exp)

    def test_ragged_table(self):
        inp = '''
| 1.1 | 1.2 | 1.3 |
| 2.1 |
| 3.1 | 3.2 |
'''
        exp = _format_table([['1.1','1.2','1.3'],
                             ['2.1','',''],
                             ['3.1','3.2','']])
        assert_format(inp, exp)

    def test_th(self):
        inp = '''
| =a= |   =   b   =   | = = c = = |
| = = |    = _e_ =    |  =_*f*_=  |
'''
        exp = '''
<table border="1">
<tr>
<th>a</th>
<th>b</th>
<th>= c =</th>
</tr>
<tr>
<th></th>
<th><i>e</i></th>
<th><i><b>f</b></i></th>
</tr>
</table>
'''
        assert_format(inp, exp.strip())

    def test_bold_in_table_cells(self):
        inp = '''
| *a* | *b* | *c* |
| *b* |  x  |  y  |
| *c* |  z  |     |

| a   | x *b* y | *b* *c* |
| *a  | b*      |         |
'''
        exp = _format_table([['<b>a</b>','<b>b</b>','<b>c</b>'],
                             ['<b>b</b>','x','y'],
                             ['<b>c</b>','z','']]) + '\n' \
            + _format_table([['a','x <b>b</b> y','<b>b</b> <b>c</b>'],
                             ['*a','b*','']])
        assert_format(inp, exp)

    def test_italic_in_table_cells(self):
        inp = '''
| _a_ | _b_ | _c_ |
| _b_ |  x  |  y  |
| _c_ |  z  |     |

| a   | x _b_ y | _b_ _c_ |
| _a  | b_      |         |
'''
        exp = _format_table([['<i>a</i>','<i>b</i>','<i>c</i>'],
                             ['<i>b</i>','x','y'],
                             ['<i>c</i>','z','']]) + '\n' \
            +  _format_table([['a','x <i>b</i> y','<i>b</i> <i>c</i>'],
                              ['_a','b_','']])
        assert_format(inp, exp)

    def test_bold_and_italic_in_table_cells(self):
        inp = '''
| *a* | *b* | *c* |
| _b_ |  x  |  y  |
| _c_ |  z  | *b* _i_ |
'''
        exp = _format_table([['<b>a</b>','<b>b</b>','<b>c</b>'],
                             ['<i>b</i>','x','y'],
                             ['<i>c</i>','z','<b>b</b> <i>i</i>']])
        assert_format(inp, exp)

    def test_link_in_table_cell(self):
        inp = '''
| 1 | http://one |
| 2 | ftp://two/ |
'''
        exp = _format_table([['1','FIRST'],
                             ['2','SECOND']]) \
            .replace('FIRST', '<a href="http://one">http://one</a>') \
            .replace('SECOND', '<a href="ftp://two/">ftp://two/</a>')
        assert_format(inp, exp)


class TestHtmlFormatHr(unittest.TestCase):

    def test_hr_is_three_or_more_hyphens(self):
        for i in range(3, 10):
            hr = '-' * i
            spaces = ' ' * i
            assert_format(hr, '<hr>')
            assert_format(spaces + hr + spaces, '<hr>')

    def test_hr_with_other_stuff_around(self):
        for inp, exp in [('---\n-', '<hr>\n<p>-</p>'),
                         ('xx\n---\nxx', '<p>xx</p>\n<hr>\n<p>xx</p>'),
                         ('xx\n\n------\n\nxx', '<p>xx</p>\n<hr>\n<p>xx</p>')]:
            assert_format(inp, exp)

    def test_multiple_hrs(self):
        assert_format('---\n---\n\n---', '<hr>\n<hr>\n<hr>')

    def test_not_hr(self):
        for inp in ['-', '--', '-- --', '...---...', '===']:
            assert_format(inp, p=True)

    def test_hr_before_and_after_table(self):
        inp = '''
---
| t | a | b | l | e |
---'''
        exp = '<hr>\n' + _format_table([['t','a','b','l','e']]) + '\n<hr>'
        assert_format(inp, exp)


class TestHtmlFormatList(unittest.TestCase):

    def test_not_a_list(self):
        for inp in ('-- item', '+ item', '* item', '-item'):
            assert_format(inp, inp, p=True)

    def test_one_item_list(self):
        assert_format('- item', '<ul>\n<li>item</li>\n</ul>')
        assert_format(' -   item', '<ul>\n<li>item</li>\n</ul>')

    def test_multi_item_list(self):
        assert_format('- 1\n  -  2\n- 3',
                      '<ul>\n<li>1</li>\n<li>2</li>\n<li>3</li>\n</ul>')

    def test_list_with_formatted_content(self):
        assert_format('- *bold* text\n- _italic_\n- [http://url|link]',
                      '<ul>\n<li><b>bold</b> text</li>\n<li><i>italic</i></li>\n'
                      '<li><a href="http://url">link</a></li>\n</ul>')

    def test_indentation_can_be_used_to_continue_list_item(self):
        assert_format('''
  outside list
- this item
  continues
  - 2nd item
    continues
 twice
''', '''\
<p>outside list</p>
<ul>
<li>this item continues</li>
<li>2nd item continues twice</li>
</ul>''')

    def test_lists_with_other_content_around(self):
        assert_format('''
before
- a
- *b*
between

- c
- d
  e
  f

---
''',  '''\
<p>before</p>
<ul>
<li>a</li>
<li><b>b</b></li>
</ul>
<p>between</p>
<ul>
<li>c</li>
<li>d e f</li>
</ul>
<hr>''')


class TestHtmlFormatPreformatted(unittest.TestCase):

    def test_single_line_block(self):
        self._assert_preformatted('| some', 'some')

    def test_block_without_any_content(self):
        self._assert_preformatted('|', '')

    def test_first_char_after_pipe_must_be_space(self):
        assert_format('|x', p=True)

    def test_multi_line_block(self):
        self._assert_preformatted('| some\n|\n| quote', 'some\n\nquote')

    def test_internal_whitespace_is_preserved(self):
        self._assert_preformatted('|   so\t\tme  ', '  so\t\tme')

    def test_spaces_before_leading_pipe_are_ignored(self):
        self._assert_preformatted(' | some', 'some')

    def test_block_mixed_with_other_content(self):
        assert_format('before block:\n| some\n| quote\nafter block',
                      '<p>before block:</p>\n<pre>\nsome\nquote\n</pre>\n<p>after block</p>')

    def test_multiple_blocks(self):
        assert_format('| some\n| quote\nbetween\n| other block\n\nafter', '''\
<pre>
some
quote
</pre>
<p>between</p>
<pre>
other block
</pre>
<p>after</p>''')

    def test_block_line_with_other_formatting(self):
        self._assert_preformatted('| _some_ formatted\n| text *here*',
                                  '<i>some</i> formatted\ntext <b>here</b>')

    def _assert_preformatted(self, inp, exp):
        assert_format(inp, '<pre>\n' + exp + '\n</pre>')


class TestHtmlFormatHeaders(unittest.TestCase):

    def test_no_header(self):
        for line in ['', 'hello', '=', '==', '====', '= =', '=  =', '==     ==',
                     '= inconsistent levels ==', '==== 4 is too many ====',
                     '=no spaces=', '=no spaces =', '= no spaces=']:
            assert_format(line, p=bool(line))

    def test_header(self):
        for line, expected in [('= My Header =', '<h2>My Header</h2>'),
                               ('== my == header ==', '<h3>my == header</h3>'),
                               ('  === ===    ===  ', '<h4>===</h4>')]:
            assert_format(line, expected)


class TestFormatTable(unittest.TestCase):
    # RIDE needs border="1" because its HTML view doesn't support CSS
    _table_start = '<table border="1">'

    def test_one_row_table(self):
        inp = [['1','2','3']]
        exp = self._table_start + '''
<tr>
<td>1</td>
<td>2</td>
<td>3</td>
</tr>
</table>'''
        assert_equal(_format_table(inp), exp)

    def test_multi_row_table(self):
        inp = [['1.1','1.2'], ['2.1','2.2'], ['3.1','3.2']]
        exp = self._table_start + '''
<tr>
<td>1.1</td>
<td>1.2</td>
</tr>
<tr>
<td>2.1</td>
<td>2.2</td>
</tr>
<tr>
<td>3.1</td>
<td>3.2</td>
</tr>
</table>'''
        assert_equal(_format_table(inp), exp)

    def test_fix_ragged_table(self):
        inp = [['1.1','1.2','1.3'], ['2.1'], ['3.1','3.2']]
        exp = self._table_start + '''
<tr>
<td>1.1</td>
<td>1.2</td>
<td>1.3</td>
</tr>
<tr>
<td>2.1</td>
<td></td>
<td></td>
</tr>
<tr>
<td>3.1</td>
<td>3.2</td>
<td></td>
</tr>
</table>'''
        assert_equal(_format_table(inp), exp)

    def test_th(self):
        inp = [['=h1.1=', '=  h  1.2   ='], ['== _h2.1_ =', '= not h 2.2']]
        exp = self._table_start + '''
<tr>
<th>h1.1</th>
<th>h  1.2</th>
</tr>
<tr>
<th>= <i>h2.1</i></th>
<td>= not h 2.2</td>
</tr>
</table>'''
        assert_equal(_format_table(inp), exp)


class TestAttributeEscape(unittest.TestCase):

    def test_nothing_to_escape(self):
        for inp in ['', 'whatever', 'nothing here, move along']:
            assert_equal(attribute_escape(inp), inp)

    def test_html_entities(self):
        for inp, exp in [('"', '&quot;'), ('<', '&lt;'), ('>', '&gt;'),
                         ('&', '&amp;'), ('&<">&', '&amp;&lt;&quot;&gt;&amp;'),
                         ('Sanity < "check"', 'Sanity &lt; &quot;check&quot;')]:
            assert_equal(attribute_escape(inp), exp)

    def test_newlines_and_tabs(self):
        for inp, exp in [('\n', '&#10;'), ('\t', '&#09;'), ('"\n\t"', '&quot;&#10;&#09;&quot;'),
                         ('N1\nN2\n\nT1\tT3\t\t\t', 'N1&#10;N2&#10;&#10;T1&#09;T3&#09;&#09;&#09;')]:
            assert_equal(attribute_escape(inp), exp)

    def test_illegal_chars_in_xml(self):
        for c in u'\x00\x08\x0B\x0C\x0E\x1F\uFFFE\uFFFF':
            assert_equal(attribute_escape(c), '')


if __name__ == '__main__':
    unittest.main()



from robot.api.deco import keyword


@keyword('No ${Ope}ration')
def no_operation(ope):
    raise AssertionError('Should not be run due to keywords with normal '
                         'arguments having higher precedence.')


@keyword('Get ${Name}')
def get_name(name):
    raise AssertionError('Should not be run due to keywords with normal '
                         'arguments having higher precedence.')

@keyword('Get ${Name} With Search Order')
def get_name_with_search_order(name):
    return "embedded"



import unittest

from robot.utils.asserts import assert_true, assert_equal
from test_jsmodelbuilders import remap
from robot.reporting.jsexecutionresult import (JsExecutionResult,
                                               _KeywordRemover, StringIndex)
from robot.reporting.jsmodelbuilders import SuiteBuilder, JsBuildingContext
from robot.result import TestSuite


class TestRemoveDataNotNeededInReport(unittest.TestCase):

    def test_remove_keywords(self):
        model = self._create_suite_model()
        expected = self._get_expected_suite_model(model)
        result = _KeywordRemover().remove_keywords(model)
        assert_equal(result, expected)
        self._verify_model_contains_no_keywords(result)

    def _create_suite_model(self):
        self.context = JsBuildingContext()
        return SuiteBuilder(self.context).build(self._get_suite())

    def _get_suite(self):
        suite = TestSuite(name='root', doc='sdoc', metadata={'m': 'v'})
        suite.setup.config(kwname='keyword')
        sub = suite.suites.create(name='suite', metadata={'a': '1', 'b': '2'})
        sub.setup.config(kwname='keyword')
        t1 = sub.tests.create(name='test', tags=['t1'])
        t1.body.create_keyword(kwname='keyword')
        t1.body.create_keyword(kwname='keyword')
        t2 = sub.tests.create(name='test', tags=['t1', 't2'])
        t2.body.create_keyword(kwname='keyword')
        return suite

    def _get_expected_suite_model(self, suite):
        suite = list(suite)
        suite[-4] = tuple(self._get_expected_suite_model(s) for s in suite[-4])
        suite[-3] = tuple(self._get_expected_test_model(t) for t in suite[-3])
        suite[-2] = ()
        return tuple(suite)

    def _get_expected_test_model(self, test):
        test = list(test)
        test[-1] = ()
        return tuple(test)

    def _verify_model_contains_no_keywords(self, model, mapped=False):
        if not mapped:
            model = remap(model, self.context.strings)
        assert_true('keyword' not in model, 'Not all keywords removed')
        for item in model:
            if isinstance(item, tuple):
                self._verify_model_contains_no_keywords(item, mapped=True)

    def test_remove_unused_strings(self):
        strings = ('', 'hei', 'hoi')
        model = (1, StringIndex(0), 42, StringIndex(2), -1, None)
        model, strings = _KeywordRemover().remove_unused_strings(model, strings)
        assert_equal(strings, ('', 'hoi'))
        assert_equal(model, (1, 0, 42, 1, -1, None))

    def test_remove_unused_strings_nested(self):
        strings = tuple(' abcde')
        model = (StringIndex(0), StringIndex(1), 2, 3, StringIndex(4), 5,
                 (0, StringIndex(1), 2, StringIndex(3), 4, 5))
        model, strings = _KeywordRemover().remove_unused_strings(model, strings)
        assert_equal(strings, tuple(' acd'))
        assert_equal(model, (0, 1, 2, 3, 3, 5, (0, 1, 2, 2, 4, 5)))

    def test_through_jsexecutionresult(self):
        suite = (0, StringIndex(1), 2, 3, 4, StringIndex(5),
                 ((0, 1, 2, StringIndex(3), 4, 5, (), (), ('suite', 'kws'), 9),),
                 ((0, 1, 2, StringIndex(3), 4, 5, ('test', 'kws')),
                  (0, StringIndex(1), 2, 3, 4, 5, ('test', 'kws'))),
                 ('suite', 'kws'), 9)
        exp_s = (0, 0, 2, 3, 4, 2,
                 ((0, 1, 2, 1, 4, 5, (), (), (), 9),),
                 ((0, 1, 2, 1, 4, 5, ()),
                  (0, 0, 2, 3, 4, 5, ())),
                 (), 9)
        result = JsExecutionResult(suite=suite, strings=tuple(' ABCDEF'),
                                   errors=(1, 2), statistics={}, basemillis=0,
                                   min_level='DEBUG')
        assert_equal(result.data['errors'], (1, 2))
        result.remove_data_not_needed_in_report()
        assert_equal(result.strings, tuple('ACE'))
        assert_equal(result.suite, exp_s)
        assert_equal(result.min_level, 'DEBUG')
        assert_true('errors' not in result.data)


if __name__ == '__main__':
    unittest.main()



import inspect
import sys
from xmlrpc.server import SimpleXMLRPCServer


# Re-implementation of `robot.api.deco.keyword`. Cannot use the real thing
# because `robot` may not installed on this interpreter.
def keyword(name=None, tags=(), types=()):
    if callable(name):
        return keyword()(name)
    def deco(func):
        func.robot_name = name
        func.robot_tags = tags
        func.robot_types = types
        return func
    return deco


class RemoteServer(SimpleXMLRPCServer):

    def __init__(self, library, port=8270, port_file=None):
        SimpleXMLRPCServer.__init__(self, ('127.0.0.1', int(port)))
        self.library = library
        self._shutdown = False
        self._register_functions()
        announce_port(self.socket, port_file)
        self.serve_forever()

    def _register_functions(self):
        self.register_function(self.get_keyword_names)
        self.register_function(self.get_keyword_arguments)
        self.register_function(self.get_keyword_tags)
        self.register_function(self.get_keyword_documentation)
        self.register_function(self.run_keyword)

    def serve_forever(self):
        while not self._shutdown:
            self.handle_request()

    def get_keyword_names(self):
        return [attr for attr in dir(self.library) if attr[0] != '_']

    def get_keyword_arguments(self, name):
        kw = getattr(self.library, name)
        args, varargs, kwargs, defaults, kwoargs, kwodefaults, _ \
            = inspect.getfullargspec(kw)
        args = args[1:]  # drop 'self'
        if defaults:
            args, names = args[:-len(defaults)], args[-len(defaults):]
            args += [f'{n}={d}' for n, d in zip(names, defaults)]
        if varargs:
            args.append(f'*{varargs}')
        if kwoargs:
            if not varargs:
                args.append('*')
            args += [self._format_kwo(arg, kwodefaults) for arg in kwoargs]
        if kwargs:
            args.append(f'**{kwargs}')
        return args

    def _format_kwo(self, arg, defaults):
        if defaults and arg in defaults:
            return f'{arg}={defaults[arg]}'
        return arg

    def get_keyword_tags(self, name):
        kw = getattr(self.library, name)
        return getattr(kw, 'robot_tags', [])

    def get_keyword_documentation(self, name):
        kw = getattr(self.library, name)
        return inspect.getdoc(kw) or ''

    def run_keyword(self, name, args, kwargs=None):
        try:
            result = getattr(self.library, name)(*args, **(kwargs or {}))
        except AssertionError as err:
            return {'status': 'FAIL', 'error': str(err)}
        else:
            return {'status': 'PASS',
                    'return': result if result is not None else ''}


class DirectResultRemoteServer(RemoteServer):

    def run_keyword(self, name, args, kwargs=None):
        try:
            return getattr(self.library, name)(*args, **(kwargs or {}))
        except SystemExit:
            self._shutdown = True
            return {'status': 'PASS'}


def announce_port(socket, port_file=None):
    port = socket.getsockname()[1]
    sys.stdout.write(f'Remote server starting on port {port}.\n')
    sys.stdout.flush()
    if port_file:
        with open(port_file, 'w') as f:
            f.write(str(port))



from robot import utils

class MyLibDir:
    
    def get_keyword_names(self):
        return ['Keyword In My Lib Dir']
    
    def run_keyword(self, name, args):
        return "Executed keyword '%s' with args %s" % (name, utils.seq2str2(args))



message_list = ['Circle is 360\u00B0',
                'Hyv\u00E4\u00E4 \u00FC\u00F6t\u00E4',
                '\u0989\u09C4 \u09F0 \u09FA \u099F \u09EB \u09EA \u09B9']

message1 = message_list[0]
message2 = message_list[1]
message3 = message_list[2]

messages = ', '.join(message_list)

sect = chr(167)
auml = chr(228)
ouml = chr(246)
uuml = chr(252)
yuml = chr(255)



import sys
from xmlrpc.server import SimpleXMLRPCServer

from remoteserver import announce_port


class Documentation(SimpleXMLRPCServer):

    def __init__(self, port=8270, port_file=None):
        SimpleXMLRPCServer.__init__(self, ('127.0.0.1', int(port)))
        self.register_function(self.get_keyword_names)
        self.register_function(self.get_keyword_documentation)
        self.register_function(self.get_keyword_arguments)
        self.register_function(self.run_keyword)
        announce_port(self.socket, port_file)
        self.serve_forever()

    def get_keyword_names(self):
        return ['Empty', 'Single', 'Multi', 'Nön-ÄSCII']

    def get_keyword_documentation(self, name):
        return {'__intro__': 'Remote library for documentation testing purposes',
                'Empty': '',
                'Single': 'Single line documentation',
                'Multi': 'Short doc\nin two lines.\n\nDoc body\nin\nthree.',
                'Nön-ÄSCII': 'Nön-ÄSCII documentation'}.get(name)

    def get_keyword_arguments(self, name):
        return {'Empty': (),
                'Single': ['arg'],
                'Multi': ['a1', 'a2=d', '*varargs']}.get(name)

    def run_keyword(self, name, args):
        return {'status': 'PASS'}


if __name__ == '__main__':
    Documentation(*sys.argv[1:])



from sys import argv, stdout, stderr
from time import sleep

timeout = float(argv[1]) if len(argv) > 1 else 1

stdout.write('start stdout\n')
stdout.flush()
stderr.write('start stderr\n')
stderr.flush()
sleep(timeout)
stdout.write('end stdout\n')
stdout.flush()
stderr.write('end stderr\n')
stderr.flush()



def importing_robot_module_directly_fails():
    try:
        import running
    except ImportError:
        pass
    else:
        raise AssertionError("Importing 'running' directly succeeded!")


def importing_robot_module_through_robot_succeeds():
    from robot import running


def importing_standard_library_directly_fails():
    try:
        import BuiltIn
    except ImportError:
        pass
    else:
        raise AssertionError("Importing 'BuiltIn' directly succeeded!")

def importing_standard_library_through_robot_libraries_succeeds():
    from robot.libraries import BuiltIn
    BuiltIn.BuiltIn().set_test_variable('${SET BY LIBRARY}', 42)



from os.path import abspath, dirname, join, normpath
import sys

curdir = dirname(abspath(__file__))
src = normpath(join(curdir, '..', '..', '..', '..', '..', 'src'))
sys.path.insert(0, src)

from robot.utils.encoding import CONSOLE_ENCODING, SYSTEM_ENCODING


config = dict(arg.split(':') for arg in sys.argv[1:])
stdout = config.get('stdout', 'hyv\xe4')
stderr = config.get('stderr', stdout)
encoding = config.get('encoding', 'ASCII')
encoding = {'CONSOLE': CONSOLE_ENCODING,
            'SYSTEM': SYSTEM_ENCODING}.get(encoding, encoding)


sys.stdout.buffer.write(stdout.encode(encoding))
sys.stderr.buffer.write(stderr.encode(encoding))



import unittest

from robot.utils.asserts import assert_equal
from robot.output.console.verbose import VerboseOutput


class TestKeywordNotification(unittest.TestCase):

    def setUp(self, markers='AUTO', isatty=True):
        self.stream = StreamStub(isatty)
        self.console = VerboseOutput(width=16, colors='off', markers=markers,
                                     stdout=self.stream, stderr=self.stream)
        self.console.start_test(Stub())

    def test_write_pass_marker(self):
        self._write_marker()
        self._verify('.')

    def test_write_fail_marker(self):
        self._write_marker('FAIL')
        self._verify('F')

    def test_multiple_markers(self):
        self._write_marker()
        self._write_marker('FAIL')
        self._write_marker('FAIL')
        self._write_marker()
        self._verify('.FF.')

    def test_maximum_number_of_markers(self):
        self._write_marker(count=8)
        self._verify('........')

    def test_more_markers_than_fit_into_status_area(self):
        self._write_marker(count=9)
        self._verify('.')
        self._write_marker(count=10)
        self._verify('...')

    def test_clear_markers_when_test_status_is_written(self):
        self._write_marker(count=5)
        self.console.end_test(Stub())
        self._verify('| PASS |\n%s\n' % ('-'*self.console._writer._width))

    def test_clear_markers_when_there_are_warnings(self):
        self._write_marker(count=5)
        self.console.message(MessageStub())
        self._verify(before='[ WARN ] Message\n')
        self._write_marker(count=2)
        self._verify(before='[ WARN ] Message\n', after='..')

    def test_markers_off(self):
        self.setUp(markers='OFF')
        self._write_marker()
        self._write_marker('FAIL')
        self._verify()

    def test_markers_on(self):
        self.setUp(markers='on', isatty=False)
        self._write_marker()
        self._write_marker('FAIL')
        self._verify('.F')

    def test_markers_auto_off(self):
        self.setUp(markers='AUTO', isatty=False)
        self._write_marker()
        self._write_marker('FAIL')
        self._verify()

    def _write_marker(self, status='PASS', count=1):
        for i in range(count):
            self.console.start_keyword(Stub())
            self.console.end_keyword(Stub(status=status))

    def _verify(self, after='', before=''):
        assert_equal(str(self.stream), '%sX :: D  %s' % (before, after))


class Stub:

    def __init__(self, name='X', doc='D', status='PASS', message=''):
        self.name = name
        self.doc = doc
        self.status = status
        self.message = message

    @property
    def passed(self):
        return self.status == 'PASS'


class MessageStub:

    def __init__(self, message='Message', level='WARN'):
        self.message = message
        self.level = level


class StreamStub:

    def __init__(self, isatty=True):
        self.buffer = []
        self.isatty = lambda: isatty

    def write(self, msg):
        self.buffer.append(msg)

    def flush(self):
        pass

    def __str__(self):
        return ''.join(self.buffer).rsplit('\r')[-1]


if __name__ == '__main__':
    unittest.main()



from helper import pretty


KEYWORDS = {
    'One Arg': ['arg'],
    'Two Args': ['first', 'second'],
    'Four Args': ['a=1', ('b', '2'), ('c', 3), ('d', 4)],
    'Defaults w/ Specials': ['a=${notvar}', 'b=\n', 'c=\\n', 'd=\\'],
    'Args & Varargs': ['a', 'b=default', '*varargs'],
    'Nön-ÄSCII names': ['nönäscii', '官话'],
}


class DynamicWithoutKwargs:

    def __init__(self, **extra):
        self.keywords = dict(KEYWORDS, **extra)

    def get_keyword_names(self):
        return self.keywords.keys()

    def run_keyword(self, kw_name, args):
        return self._pretty(*args)

    def _pretty(self, *args, **kwargs):
        return pretty(*args, **kwargs)

    def get_keyword_arguments(self, kw_name):
        return self.keywords[kw_name]



from CustomConverters import Number, string_to_int


class CustomConvertersWithDynamicLibrary:
    ROBOT_LIBRARY_CONVERTERS = {Number: string_to_int}

    def get_keyword_names(self):
        return ['dynamic keyword']

    def run_keyword(self, name, args, named):
        self._validate(*args, **named)

    def _validate(self, argument, expected):
        assert argument == expected

    def get_keyword_arguments(self, name):
        return ['argument', 'expected']

    def get_keyword_types(self, name):
        return [Number, int]



from io import StringIO
import unittest

from robot.running.model import TestSuite, Import
from robot.utils.asserts import assert_equal, assert_raises_with_msg


def run(suite, **config):
    result = suite.run(output=None, log=None, report=None,
                       stdout=StringIO(), stderr=StringIO(), **config)
    return result.suite


def assert_suite(suite, name, status, message='', tests=1):
    assert_equal(suite.name, name)
    assert_equal(suite.status, status)
    assert_equal(suite.message, message)
    assert_equal(len(suite.tests), tests)


def assert_test(test, name, status, tags=(), msg=''):
    assert_equal(test.name, name)
    assert_equal(test.status, status)
    assert_equal(test.message, msg)
    assert_equal(tuple(test.tags), tags)


class TestImports(unittest.TestCase):

    def run_and_check_pass(self, suite):
        result = run(suite)
        try:
            assert_suite(result, 'Suite', 'PASS')
            assert_test(result.tests[0], 'Test', 'PASS')
        except AssertionError as e:
            # Something failed. Let's print more info.
            full_msg = ["Expected and obtained don't match. Test messages:"]
            for test in result.tests:
                full_msg.append('%s: %s' % (test, test.message))
            raise AssertionError('\n'.join(full_msg)) from e

    def test_create(self):
        suite = TestSuite(name='Suite')
        suite.resource.imports.create('Library', 'OperatingSystem')
        suite.resource.imports.create('RESOURCE', 'test.resource')
        suite.resource.imports.create(type='LibRary', name='String')
        test = suite.tests.create(name='Test')
        test.body.create_keyword('Directory Should Exist', args=['.'])
        test.body.create_keyword('My Test Keyword')
        test.body.create_keyword('Convert To Lower Case', args=['ROBOT'])
        self.run_and_check_pass(suite)

    def test_library(self):
        suite = TestSuite(name='Suite')
        suite.resource.imports.library('OperatingSystem')
        suite.tests.create(name='Test').body.create_keyword('Directory Should Exist',
                                                            args=['.'])
        self.run_and_check_pass(suite)

    def test_resource(self):
        suite = TestSuite(name='Suite')
        suite.resource.imports.resource('test.resource')
        suite.tests.create(name='Test').body.create_keyword('My Test Keyword')
        assert_equal(suite.tests[0].body[0].name, 'My Test Keyword')
        self.run_and_check_pass(suite)

    def test_variables(self):
        suite = TestSuite(name='Suite')
        suite.resource.imports.variables('variables_file.py')
        suite.tests.create(name='Test').body.create_keyword(
            'Should Be Equal As Strings',
            args=['${MY_VARIABLE}', 'An example string']
        )
        self.run_and_check_pass(suite)

    def test_invalid_type(self):
        assert_raises_with_msg(ValueError,
                               "Invalid import type: Expected 'LIBRARY', 'RESOURCE' "
                               "or 'VARIABLES', got 'INVALIDTYPE'.",
                               TestSuite().resource.imports.create,
                               'InvalidType', 'Name')

    def test_repr(self):
        assert_equal(repr(Import(Import.LIBRARY, 'X')),
                     "robot.running.Import(type='LIBRARY', name='X')")
        assert_equal(repr(Import(Import.LIBRARY, 'X', ['a'], 'A')),
                     "robot.running.Import(type='LIBRARY', name='X', args=('a',), alias='A')")
        assert_equal(repr(Import(Import.RESOURCE, 'X')),
                     "robot.running.Import(type='RESOURCE', name='X')")
        assert_equal(repr(Import(Import.VARIABLES, '')),
                     "robot.running.Import(type='VARIABLES', name='')")


if __name__ == '__main__':
    unittest.main()



import logging
from robot.api import logger


ROBOT_LISTENER_API_VERSION = 2


def get_logging_listener_method(name):
    def listener_method(*args):
        if name in ['message', 'log_message']:
            msg = args[0]
            message = f"{name}: {msg['level']} {msg['message']}"
        elif name == 'start_keyword':
            message = f"start {args[1]['type']}".lower()
        elif name == 'end_keyword':
            message = f"end {args[1]['type']}".lower()
        else:
            message = name
        logging.info(message)
        logger.warn(message)
    return listener_method


for name in ['start_suite', 'end_suite', 'start_test', 'end_test',
             'start_keyword', 'end_keyword', 'log_message', 'message',
             'output_file', 'log_file', 'report_file', 'debug_file', 'close']:
    globals()[name] = get_logging_listener_method(name)



import ast

from robot.parsing import ModelTransformer
from robot.parsing.model.blocks import Container
from robot.parsing.model.statements import Statement

from robot.utils.asserts import assert_equal


def assert_model(model, expected, **expected_attrs):
    if type(model) is not type(expected):
        raise AssertionError('Incompatible types:\n%s\n%s'
                             % (dump_model(model), dump_model(expected)))
    if isinstance(model, list):
        assert_equal(len(model), len(expected),
                     '%r != %r' % (model, expected), values=False)
        for m, e in zip(model, expected):
            assert_model(m, e)
    elif isinstance(model, Container):
        assert_block(model, expected, expected_attrs)
    elif isinstance(model, Statement):
        assert_statement(model, expected)
    elif model is None and expected is None:
        pass
    else:
        raise AssertionError('Incompatible children:\n%r\n%r' % (model, expected))


def dump_model(model):
    if isinstance(model, ast.AST):
        return ast.dump(model)
    elif isinstance(model, (list, tuple)):
        return [dump_model(m) for m in model]
    elif model is None:
        return 'None'
    else:
        raise TypeError('Invalid model %r' % model)


def assert_block(model, expected, expected_attrs):
    assert_equal(model._fields, expected._fields)
    for field in expected._fields:
        assert_model(getattr(model, field), getattr(expected, field))
    for attr in expected._attributes:
        exp = expected_attrs.get(attr, getattr(expected, attr))
        assert_equal(getattr(model, attr), exp)


def assert_statement(model, expected):
    assert_equal(model._fields, ('type', 'tokens'))
    assert_equal(model.type, expected.type)
    assert_equal(len(model.tokens), len(expected.tokens))
    for m, e in zip(model.tokens, expected.tokens):
        assert_equal(m, e, formatter=repr)
    assert_equal(model._attributes, ('lineno', 'col_offset', 'end_lineno',
                                     'end_col_offset', 'errors'))
    assert_equal(model.lineno, expected.tokens[0].lineno)
    assert_equal(model.col_offset, expected.tokens[0].col_offset)
    assert_equal(model.end_lineno, expected.tokens[-1].lineno)
    assert_equal(model.end_col_offset, expected.tokens[-1].end_col_offset)
    assert_equal(model.errors, expected.errors)


def remove_non_data(model):
    RemoveNonDataTokensVisitor().visit(model)


class RemoveNonDataTokensVisitor(ModelTransformer):

    def visit_Statement(self, node):
        node.tokens = node.data_tokens
        return node

    def visit_EmptyLine(self, none):
        return None

    def visit_Comment(self, node):
        return None



from robot.api.deco import keyword


@keyword('${x} in library')
def x_in_library(x):
    assert x == 'x'


@keyword('${x} and ${y} in library')
def x_and_y_in_library(x, y):
    assert x == 'x'
    assert y == 'y'


@keyword('${y:y} in library')
def y_in_library(y):
    assert False


@keyword('${match} in ${both} libraries')
def match_in_both_libraries(match, both):
    assert False


@keyword('Best ${match} in ${one of} libraries')
def best_match_in_one_of_libraries(match, one_of):
    assert match == 'match'
    assert one_of == 'one of'

@keyword('Follow search ${disorder} in libraries')
def follow_search_order_in_libraries(disorder):
    assert disorder == 'disorder should not happen'

@keyword('Unresolvable conflict in library')
def unresolvable_conflict_in_library():
    assert False



from robot.libraries.BuiltIn import BuiltIn, register_run_keyword


def run_keyword_function(name, *args):
    return BuiltIn().run_keyword(name, *args)


register_run_keyword(__name__, 'run_keyword_function', 1)

def run_keyword_without_keyword(*args):
    return BuiltIn().run_keyword(r'\\Log Many', *args)

register_run_keyword(__name__, 'run_keyword_without_keyword', 0)



import time
from robot.api import logger


def log_with_all_levels():
    for level in 'trace debug info warn error'.split():
        msg = '%s msg' % level
        logger.write(msg+' 1', level)
        getattr(logger, level)(msg+' 2', html=False)


def write(message, level):
    logger.write(message, level)


def log_messages_different_time():
    logger.info('First message')
    time.sleep(0.1)
    logger.info('Second message 0.1 sec later')


def log_html():
    logger.write('<b>debug</b>', level='DEBUG', html=True)
    logger.info('<b>info</b>', html=True)
    logger.warn('<b>warn</b>', html=True)


def write_messages_to_console():
    logger.console('To console only')
    logger.console('To console ', newline=False)
    logger.console('in two parts')
    logger.info('To log and console', also_console=True)


def log_non_strings():
    logger.info(42)
    logger.warn(True)


def log_callable():
    logger.info(log_callable)



class MyObject:

    def __init__(self, value):
        self.value = value

    def __int__(self):
        return 42 // self.value

    def __str__(self):
        return 'MyObject'


def get_variables():
    return {'object': MyObject(1),
            'object_failing': MyObject(0)}



#!/usr/bin/env python3

"""A script for running Robot Framework's own acceptance tests.

Usage:  atest/run.py [--interpreter name] [--schema-validation [options] [data]

`data` is path (or paths) of the file or directory under the `atest/robot`
folder to execute. If `data` is not given, all tests except for tests tagged
with `no-ci` are executed.

Available `options` are the same that can be used with Robot Framework.
See its help (e.g. `robot --help`) for more information.

By default, uses the same Python interpreter for running tests that is used
for running this script. That can be changed by using the `--interpreter`
(`-I`) option. It can be the name of the interpreter like `pypy3` or a path
to the selected interpreter like `/usr/bin/python39`. If the interpreter
itself needs arguments, the interpreter and its arguments need to be quoted
like `"py -3.9"`.

To enable schema validation for all suites, use `--schema-validation` (`-S`)
option. This is same as setting `ATEST_VALIDATE_OUTPUT` environment variable
to `TRUE`.

Examples:
$ atest/run.py
$ atest/run.py --exclude no-ci atest/robot/standard_libraries
$ atest/run.py --interpreter pypy3

The results of the test execution are written into an interpreter specific
directory under the `atest/results` directory. Temporary outputs created
during the execution are created under the system temporary directory.
"""

import argparse
import os
from pathlib import Path
import shutil
import signal
import subprocess
import sys
import tempfile

from interpreter import Interpreter


CURDIR = Path(__file__).parent
ARGUMENTS = '''
--doc Robot Framework acceptance tests
--metadata interpreter:{interpreter}
--variable-file {variable_file};{interpreter.path};{interpreter.name};{interpreter.version}
--pythonpath {pythonpath}
--output-dir {outputdir}
--splitlog
--console dotted
--console-width 100
--suite-stat-Level 3
--log NONE
--report NONE
'''.strip()


def atests(interpreter, arguments, schema_validation=False):
    try:
        interpreter = Interpreter(interpreter)
    except ValueError as err:
        sys.exit(err)
    outputdir, tempdir = _get_directories(interpreter)
    arguments = list(_get_arguments(interpreter, outputdir)) + list(arguments)
    rc = _run(arguments, tempdir, interpreter, schema_validation)
    _rebot(rc, outputdir)


def _get_directories(interpreter):
    name = interpreter.output_name
    outputdir = CURDIR / 'results' / name
    tempdir = Path(tempfile.gettempdir()) / 'robotatest' / name
    if outputdir.exists():
        shutil.rmtree(outputdir)
    if tempdir.exists():
        shutil.rmtree(tempdir)
    os.makedirs(tempdir)
    return outputdir, tempdir


def _get_arguments(interpreter, outputdir):
    arguments = ARGUMENTS.format(interpreter=interpreter,
                                 variable_file=CURDIR / 'interpreter.py',
                                 pythonpath=CURDIR / 'resources',
                                 outputdir=outputdir)
    for line in arguments.splitlines():
        yield from line.split(' ', 1)
    for exclude in interpreter.excludes:
        yield '--exclude'
        yield exclude


def _run(args, tempdir, interpreter, schema_validation):
    command = [sys.executable, str(CURDIR.parent / 'src/robot/run.py')] + args
    environ = dict(os.environ,
                   TEMPDIR=str(tempdir),
                   PYTHONCASEOK='True',
                   PYTHONIOENCODING='')
    if schema_validation:
        environ['ATEST_VALIDATE_OUTPUT'] = 'TRUE'
    print('%s\n%s\n' % (interpreter, '-' * len(str(interpreter))))
    print('Running command:\n%s\n' % ' '.join(command))
    sys.stdout.flush()
    signal.signal(signal.SIGINT, signal.SIG_IGN)
    return subprocess.call(command, env=environ)


def _rebot(rc, outputdir):
    if rc == 0:
        print('All tests passed, not generating log and report.')
    elif rc < 251:
        command = [sys.executable, str(CURDIR.parent / 'src/robot/rebot.py'),
                   '--output-dir', str(outputdir), str(outputdir / 'output.xml')]
        subprocess.call(command)


if __name__ == '__main__':
    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('-I', '--interpreter', default=sys.executable)
    parser.add_argument('-S', '--schema-validation', action='store_true')
    parser.add_argument('-h', '--help', action='store_true')
    options, robot_args = parser.parse_known_args()
    if not robot_args or not Path(robot_args[-1]).exists():
        robot_args += ['--exclude', 'no-ci', str(CURDIR/'robot')]
    if options.help:
        print(__doc__)
        rc = 251
    else:
        rc = atests(options.interpreter, robot_args, options.schema_validation)
    sys.exit(rc)



from robot.libraries.BuiltIn import BuiltIn


class LogLevels:
    ROBOT_LISTENER_API_VERSION = 2

    def __init__(self):
        self.ROBOT_LIBRARY_LISTENER = self
        self.messages = []

    def _log_message(self, msg):
        self.messages.append('%s: %s' % (msg['level'], msg['message']))

    def logged_messages_should_be(self, *expected):
        BuiltIn().should_be_equal('\n'.join(self.messages), '\n'.join(expected))



from robot.api.deco import not_keyword


class HybridWithNotKeywordDecorator:

    def get_keyword_names(self):
        return ['exposed_in_hybrid', 'not_exposed_in_hybrid']

    def exposed_in_hybrid(self):
        pass

    @not_keyword
    def not_exposed_in_hybrid(self):
        pass



LIST = ['spam', 'eggs', 21]


class _Extended:
    list = LIST
    string = 'not a list'
    def __getitem__(self, item):
        return LIST

EXTENDED = _Extended()


class _Iterable:
    def __iter__(self):
        return iter(LIST)

ITERABLE = _Iterable()



from robot.api.deco import keyword


def defined_twice():
    1/0

@keyword('Defined twice')
def this_time_using_custom_name():
    2/0

def defined_thrice():
    1/0

def definedThrice():
    2/0

def Defined_Thrice():
    3/0

@keyword('Embedded ${arguments} twice')
def embedded1(arg):
    1/0

@keyword('Embedded ${arguments match} TWICE')
def embedded2(arg):
    2/0



from numbers import Rational
from typing import List, Optional, Union
try:
    from typing import TypedDict
except ImportError:
    from typing_extensions import TypedDict


class MyObject:
    pass


class AnotherObject:
    pass


class BadRationalMeta(type(Rational)):
    def __instancecheck__(self, instance):
        raise TypeError('Bang!')


class BadRational(Rational, metaclass=BadRationalMeta):
    pass


def create_my_object():
    return MyObject()


def union_of_int_float_and_string(argument: Union[int, float, str], expected):
    assert argument == expected


def union_of_int_and_float(argument: Union[int, float], expected=object()):
    assert argument == expected


def union_with_int_and_none(argument: Union[int, None], expected=object()):
    assert argument == expected


def union_with_int_none_and_str(argument: Union[int, None, str], expected):
    assert argument == expected


def union_with_abc(argument: Union[Rational, None], expected):
    assert argument == expected


def union_with_str_and_abc(argument: Union[str, Rational], expected):
    assert argument == expected


def union_with_subscripted_generics(argument: Union[List[int], int], expected=object()):
    assert argument == eval(expected), '%r != %s' % (argument, expected)


def union_with_subscripted_generics_and_str(argument: Union[List[str], str], expected):
    assert argument == eval(expected), '%r != %s' % (argument, expected)


def union_with_typeddict(argument: Union[TypedDict('X', x=int), None], expected):
    assert argument == eval(expected), '%r != %s' % (argument, expected)


def union_with_item_not_liking_isinstance(argument: Union[BadRational, int], expected):
    assert argument == expected, '%r != %r' % (argument, expected)


def unrecognized_type(argument: Union[MyObject, str], expected_type):
    assert type(argument).__name__ == expected_type


def only_unrecognized_types(argument: Union[MyObject, AnotherObject], expected_type):
    assert type(argument).__name__ == expected_type


def tuple_of_int_float_and_string(argument: (int, float, str), expected):
    assert argument == expected


def tuple_of_int_and_float(argument: (int, float), expected=object()):
    assert argument == expected


def optional_argument(argument: Optional[int], expected):
    assert argument == expected


def optional_argument_with_default(argument: Optional[float] = None, expected=object()):
    assert argument == expected


def optional_string_with_none_default(argument: Optional[str] = None, expected=object()):
    assert argument == expected


def string_with_none_default(argument: str = None, expected=object()):
    assert argument == expected


def union_with_string_first(argument: Union[str, None], expected):
    assert argument == expected


def incompatible_default(argument: Union[None, int] = 1.1, expected=object()):
    assert argument == expected


def unrecognized_type_with_incompatible_default(argument: Union[MyObject, int] = 1.1,
                                                expected=object()):
    assert argument == expected


def union_with_invalid_types(argument: Union['nonex', 'references'], expected):
    assert argument == expected


def tuple_with_invalid_types(argument: ('invalid', 666), expected):
    assert argument == expected


def union_without_types(argument: Union):
    assert False


def empty_tuple(argument: ()):
    assert False



import sys


class listenerlibrary:
    ROBOT_LISTENER_API_VERSION = 2
    ROBOT_LIBRARY_SCOPE = "TEST CASE"

    def __init__(self):
        self.ROBOT_LIBRARY_LISTENER = self
        self.events = []
        self.level = 'suite'

    def get_events(self):
        return self.events[:]

    def _start_suite(self, name, attrs):
        self.events.append('Start suite: %s' % name)

    def endSuite(self, name, attrs):
        self.events.append('End suite: %s' % name)

    def _start_test(self, name, attrs):
        self.events.append('Start test: %s' % name)
        self.level = 'test'

    def end_test(self, name, attrs):
        self.events.append('End test: %s' % name)

    def _startKeyword(self, name, attrs):
        self.events.append('Start kw: %s' % name)

    def _end_keyword(self, name, attrs):
        self.events.append('End kw: %s' % name)

    def _close(self):
        if self.ROBOT_LIBRARY_SCOPE == 'TEST CASE':
            level = ' (%s)' % self.level
        else:
            level = ''
        sys.__stderr__.write("CLOSING %s%s\n" % (self.ROBOT_LIBRARY_SCOPE, level))

    def events_should_be(self, *expected):
        self._assert(self.events == list(expected),
                     'Expected events:\n%s\n\nActual events:\n%s'
                     % (self._format(expected), self._format(self.events)))

    def events_should_be_empty(self):
        self._assert(not self.events,
                     'Expected no events, got:\n%s' % self._format(self.events))

    def _assert(self, condition, message):
        assert condition, message

    def _format(self, events):
        return '\n'.join(events)



# Imports needed for evaluating expected result.
from datetime import datetime, date, timedelta
from decimal import Decimal

from robot.api.deco import keyword


@keyword(types=['Integer'])
def integer(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['INT'])
def int_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': 'lOnG'})
def long_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types={'argument': 'Float'})
def float_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['Double'])
def double(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['DECIMAL'])
def decimal(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['Boolean'])
def boolean(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['Bool'])
def bool_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['String'])
def string(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['BYTES'])
def bytes_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['ByteArray'])
def bytearray_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['DateTime'])
def datetime_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['Date'])
def date_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['TimeDelta'])
def timedelta_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['List'])
def list_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['TUPLE'])
def tuple_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['Dictionary'])
def dictionary(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['Dict'])
def dict_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['Map'])
def map_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['Set'])
def set_(argument, expected=None):
    _validate_type(argument, expected)


@keyword(types=['FrozenSet'])
def frozenset_(argument, expected=None):
    _validate_type(argument, expected)


def _validate_type(argument, expected):
    if isinstance(expected, str):
        expected = eval(expected)
    if argument != expected or type(argument) != type(expected):
        raise AssertionError('%r (%s) != %r (%s)'
                             % (argument, type(argument).__name__,
                                expected, type(expected).__name__))



import sys

class listenerlibrary3:
    ROBOT_LISTENER_API_VERSION = 3
    ROBOT_LIBRARY_SCOPE = "TEST CASE"

    def __init__(self):
        self.ROBOT_LIBRARY_LISTENER = self

    def start_suite(self, data, result):
        result.doc = (result.doc + ' [start suite]').strip()
        result.metadata['suite'] = '[start]'
        result.metadata['tests'] = ''
        assert len(data.tests) == 2
        assert len(result.tests) == 0
        data.tests.create(name='New')

    def end_suite(self, data, result):
        assert len(data.tests) == 3
        assert len(result.tests) == 3
        assert result.doc.endswith('[start suite]')
        assert result.metadata['suite'] == '[start]'
        result.name += ' [end suite]'
        result.doc += ' [end suite]'
        result.metadata['suite'] += ' [end]'

    def start_test(self, data, result):
        result.doc = (result.doc + ' [start test]').strip()
        result.tags.add('[start]')
        result.message = 'Message: [start]'
        result.parent.metadata['tests'] += 'x'
        data.body.create_keyword('No Operation')

    def end_test(self, data, result):
        result.doc += ' [end test]'
        result.tags.add('[end]')
        result.passed = not result.passed
        result.message += ' [end]'

    def log_message(self, msg):
        msg.message += ' [log_message]'
        msg.timestamp = '20151216 15:51:20.141'

    def foo(self):
        print("*WARN* Foo")

    def message(self, msg):
        msg.message += ' [message]'
        msg.timestamp = '20151216 15:51:20.141'

    def close(self):
        sys.__stderr__.write('CLOSING Listener library 3\n')



from collections import OrderedDict
import os
import unittest
import tempfile

from robot. errors import DataError
from robot.utils import ET, ETSource, XmlWriter
from robot.utils.asserts import assert_equal, assert_raises, assert_true

PATH = os.path.join(tempfile.gettempdir(), 'test_xmlwriter.xml')


class XmlWriterWithoutPreamble(XmlWriter):

    def _preamble(self):
        pass


class TestXmlWriter(unittest.TestCase):

    def setUp(self):
        self.writer = XmlWriterWithoutPreamble(PATH)

    def tearDown(self):
        self.writer.close()
        os.remove(PATH)

    def test_write_element_in_pieces(self):
        self.writer.start('name', {'attr': 'value'}, newline=False)
        self.writer.content('Some content here!!')
        self.writer.end('name')
        self._verify_node(None, 'name', 'Some content here!!', {'attr': 'value'})
        self._verify_content('<name attr="value">Some content here!!</name>\n')

    def test_calling_content_multiple_times(self):
        self.writer.start('element', newline=False)
        self.writer.content('Hello world!\n')
        self.writer.content('Hi again!')
        self.writer.content('\tMy name is John')
        self.writer.end('element')
        self._verify_node(None, 'element', 'Hello world!\nHi again!\tMy name is John')
        self._verify_content('<element>Hello world!\nHi again!\tMy name is John</element>\n')

    def test_write_element(self):
        self.writer.element('elem', 'Node\n content',
                            OrderedDict([('a', '1'), ('b', '2'), ('c', '3')]))
        self._verify_node(None, 'elem', 'Node\n content', {'a': '1', 'b': '2', 'c': '3'})
        self._verify_content('<elem a="1" b="2" c="3">Node\n content</elem>\n')

    def test_element_without_content_is_self_closing(self):
        self.writer.element('elem')
        self._verify_node(None, 'elem')
        self._verify_content('<elem/>\n')

    def test_element_with_empty_string_content_is_self_closing(self):
        self.writer.element('elem', '')
        self._verify_node(None, 'elem')
        self._verify_content('<elem/>\n')

    def test_element_with_attributes_but_without_content_is_self_closing(self):
        self.writer.element('elem', attrs={'attr': 'value'})
        self._verify_node(None, 'elem', attrs={'attr': 'value'})
        self._verify_content('<elem attr="value"/>\n')

    def test_write_many_elements(self):
        self.writer.start('root', {'version': 'test'})
        self.writer.start('child1', {'my-attr': 'my value'})
        self.writer.element('leaf1.1', 'leaf content', {'type': 'kw'})
        self.writer.element('leaf1.2')
        self.writer.end('child1')
        self.writer.element('child2', attrs={'class': 'foo'})
        self.writer.end('root')
        root = self._get_root()
        self._verify_node(root, 'root', attrs={'version': 'test'})
        self._verify_node(root.find('child1'), 'child1', attrs={'my-attr': 'my value'})
        self._verify_node(root.find('child1/leaf1.1'), 'leaf1.1',
                          'leaf content', {'type': 'kw'})
        self._verify_node(root.find('child1/leaf1.2'), 'leaf1.2')
        self._verify_node(root.find('child2'), 'child2', attrs={'class': 'foo'})

    def test_newline_insertion(self):
        self.writer.start('root')
        self.writer.start('suite', {'type': 'directory_suite'})
        self.writer.element('test', attrs={'name': 'my_test'}, newline=False)
        self.writer.element('test', attrs={'name': 'my_2nd_test'})
        self.writer.end('suite', False)
        self.writer.start('suite', {'name': 'another suite'}, newline=False)
        self.writer.content('Suite 2 content')
        self.writer.end('suite')
        self.writer.end('root')
        content = self._get_content()
        lines = [line for line in content.splitlines() if line != '\n']
        assert_equal(len(lines), 5)

    def test_none_content(self):
        self.writer.element(u'robot-log', None)
        self._verify_node(None, 'robot-log')

    def test_none_and_empty_attrs(self):
        self.writer.element('foo', attrs={'empty': '', 'none': None})
        self._verify_node(None, 'foo', attrs={'empty': '', 'none': ''})

    def test_content_with_invalid_command_char(self):
        self.writer.element('robot-log', '\033[31m\033[32m\033[33m\033[m')
        self._verify_node(None, 'robot-log', '[31m[32m[33m[m')

    def test_content_with_invalid_command_char_unicode(self):
        self.writer.element('robot-log', u'\x1b[31m\x1b[32m\x1b[33m\x1b[m')
        self._verify_node(None, 'robot-log', '[31m[32m[33m[m')

    def test_content_with_non_ascii(self):
        self.writer.start('root')
        self.writer.element(u'e', u'Circle is 360\xB0')
        self.writer.element(u'f', u'Hyv\xE4\xE4 \xFC\xF6t\xE4')
        self.writer.end('root')
        root = self._get_root()
        self._verify_node(root.find('e'), 'e', u'Circle is 360\xB0')
        self._verify_node(root.find('f'), 'f', u'Hyv\xE4\xE4 \xFC\xF6t\xE4')

    def test_content_with_entities(self):
        self.writer.element('I', 'Me, Myself & I > you')
        self._verify_content('<I>Me, Myself &amp; I &gt; you</I>\n')

    def test_remove_illegal_chars(self):
        assert_equal(self.writer._escape(u'\x1b[31m'), '[31m')
        assert_equal(self.writer._escape(u'\x00'), '')

    def test_dataerror_when_file_is_invalid(self):
        err = assert_raises(DataError, XmlWriter, os.path.dirname(__file__))
        assert_true(err.message.startswith('Opening file'))

    def test_dataerror_when_file_is_invalid_contains_optional_usage(self):
        err = assert_raises(DataError, XmlWriter, os.path.dirname(__file__),
                            usage='testing')
        assert_true(err.message.startswith('Opening testing file'))

    def test_dont_write_empty(self):
        self.tearDown()
        self.writer = XmlWriterWithoutPreamble(PATH, write_empty=False)
        self.writer.element('e0')
        self.writer.element('e1', content='', attrs={})
        self.writer.element('e2', attrs={'empty': '', 'None': None})
        self.writer.element('e3', attrs={'empty': '', 'value': 'value'})
        assert_equal(self._get_content(), '<e3 value="value"/>\n')

    def _verify_node(self, node, name, text=None, attrs={}):
        if node is None:
            node = self._get_root()
        assert_equal(node.tag, name)
        if text is not None:
            assert_equal(node.text, text)
        assert_equal(node.attrib, attrs)

    def _verify_content(self, expected):
        content = self._get_content()
        assert_equal(expected, content)

    def _get_root(self):
        self.writer.close()
        with ETSource(PATH) as source:
            return ET.parse(source).getroot()

    def _get_content(self):
        self.writer.close()
        with open(PATH) as f:
            return f.read()


if __name__ == '__main__':
    unittest.main()



import re
import unittest

from robot.utils import (classproperty, parse_re_flags, plural_or_not, printable_name,
                         seq2str, test_or_task)
from robot.utils.asserts import assert_equal, assert_raises, assert_raises_with_msg


class TestSeg2Str(unittest.TestCase):

    def _verify(self, input, expected, **config):
        assert_equal(seq2str(input, **config), expected)

    def test_empty(self):
        for seq in [[], (), set()]:
            self._verify(seq, '')

    def test_one_or_more(self):
        for seq, expected in [(['One'], "'One'"),
                              (['1', '2'], "'1' and '2'"),
                              (['a', 'b', 'c', 'd'], "'a', 'b', 'c' and 'd'"),
                              ([u'Unicode', u'ASCII'], "'Unicode' and 'ASCII'")]:
            self._verify(seq, expected)

    def test_non_ascii_unicode(self):
        self._verify([u'hyv\xe4'], u"'hyv\xe4'")

    def test_ascii_bytes(self):
        self._verify([b'ascii'], "'ascii'")

    def test_non_ascii_bytes(self):
        self._verify([b'non-\xe4scii'], "'non-\\xe4scii'")

    def test_other_objects(self):
        self._verify([None, 1, True], "'None', '1' and 'True'")


class TestPrintableName(unittest.TestCase):

    def test_printable_name(self):
        for inp, exp in [('simple', 'Simple'),
                         ('ALLCAPS', 'ALLCAPS'),
                         ('name with spaces', 'Name With Spaces'),
                         ('more   spaces', 'More Spaces'),
                         ('  leading and trailing  ', 'Leading And Trailing'),
                         ('  12number34  ', '12number34'),
                         ('Cases AND spaces', 'Cases AND Spaces'),
                         ('under_Score_name', 'Under_Score_name'),
                         ('camelCaseName', 'CamelCaseName'),
                         ('with89numbers', 'With89numbers'),
                         ('with 89 numbers', 'With 89 Numbers'),
                         ('with 89_numbers', 'With 89_numbers'),
                         ('', '')]:
            assert_equal(printable_name(inp), exp)

    def test_printable_name_with_code_style(self):
        for inp, exp in [('simple', 'Simple'),
                         ('ALLCAPS', 'ALLCAPS'),
                         ('name with spaces', 'Name With Spaces'),
                         ('    more   spaces    ', 'More Spaces'),
                         ('under_score_name', 'Under Score Name'),
                         ('under__score and spaces', 'Under Score And Spaces'),
                         ('__leading and trailing_ __', 'Leading And Trailing'),
                         ('__12number34__', '12 Number 34'),
                         ('miXed_CAPS_nAMe', 'MiXed CAPS NAMe'),
                         ('with 89_numbers', 'With 89 Numbers'),
                         ('camelCaseName', 'Camel Case Name'),
                         ('mixedCAPSCamelName', 'Mixed CAPS Camel Name'),
                         ('camelCaseWithDigit1', 'Camel Case With Digit 1'),
                         ('teamX', 'Team X'),
                         ('name42WithNumbers666', 'Name 42 With Numbers 666'),
                         ('name42WITHNumbers666', 'Name 42 WITH Numbers 666'),
                         ('12more34numbers', '12 More 34 Numbers'),
                         ('2KW', '2 KW'),
                         ('KW2', 'KW 2'),
                         ('xKW', 'X KW'),
                         ('KWx', 'K Wx'),
                         (':KW', ':KW'),
                         ('KW:', 'KW:'),
                         ('foo-bar', 'Foo-bar'),
                         ('Foo-b:a;r!', 'Foo-b:a;r!'),
                         ('Foo-B:A;R!', 'Foo-B:A;R!'),
                         ('', '')]:
            assert_equal(printable_name(inp, code_style=True), exp)


class TestPluralOrNot(unittest.TestCase):

    def test_plural_or_not(self):
        for singular in [1, -1, (2,), ['foo'], {'key': 'value'}, 'x']:
            assert_equal(plural_or_not(singular), '')
        for plural in [0, 2, -2, 42,
                       (), [], {},
                       (1, 2, 3), ['a', 'b'], {'a': 1, 'b': 2},
                       '', 'xx', 'Hello, world!']:
            assert_equal(plural_or_not(plural), 's')


class TestTestOrTask(unittest.TestCase):

    def test_no_match(self):
        for inp in ['', 'No match', 'No {match}', '{No} {task} {match}']:
            assert_equal(test_or_task(inp, rpa=False), inp)
            assert_equal(test_or_task(inp, rpa=True), inp)

    def test_match(self):
        for test, task in [('test', 'task'),
                           ('Test', 'Task'),
                           ('TEST', 'TASK'),
                           ('tESt', 'tASk')]:
            inp = '{%s}' % test
            assert_equal(test_or_task(inp, rpa=False), test)
            assert_equal(test_or_task(inp, rpa=True), task)

    def test_multiple_matches(self):
        assert_equal(test_or_task('Contains {test}, {TEST} and {TesT}', False),
                     'Contains test, TEST and TesT')
        assert_equal(test_or_task('Contains {test}, {TEST} and {TesT}', True),
                     'Contains task, TASK and TasK')

    def test_test_without_curlies(self):
        for test, task in [('test', 'task'),
                           ('Test', 'Task'),
                           ('TEST', 'TASK'),
                           ('tESt', 'tASk')]:
            assert_equal(test_or_task(test, rpa=False), test)
            assert_equal(test_or_task(test, rpa=True), task)


class TestParseReFlags(unittest.TestCase):

    def test_parse(self):
        for inp, exp in [('DOTALL', re.DOTALL),
                         ('I', re.I),
                         ('IGNORECASE|dotall', re.IGNORECASE | re.DOTALL),
                         (' MULTILINE ', re.MULTILINE)]:
            assert_equal(parse_re_flags(inp), exp)

    def test_parse_empty(self):
        for inp in ['', None]:
            assert_equal(parse_re_flags(inp), 0)

    def test_parse_negative(self):
        for inp, exp_msg in [('foo', 'Unknown regexp flag: foo'),
                             ('IGNORECASE|foo', 'Unknown regexp flag: foo'),
                             ('compile', 'Unknown regexp flag: compile')]:
            assert_raises_with_msg(ValueError, exp_msg, parse_re_flags, inp)


class TestClassProperty(unittest.TestCase):

    def setUp(self):
        class Class:
            @classproperty
            def p(cls):
                assert cls is Class
                return 1
        self.cls = Class

    def test_get_from_class(self):
        assert self.cls.p == 1

    def test_get_from_instance(self):
        assert self.cls().p == 1

    def test_set_in_class_overrides(self):
        # This cannot be avoided without using metaclasses.
        self.cls.p = 2
        assert self.cls.p == 2
        assert self.cls().p == 2

    def test_set_in_instance_fails(self):
        assert_raises(AttributeError, setattr, self.cls(), 'p', 2)

    def test_cannot_have_setter(self):
        code = '''
class Class:
    @classproperty
    def p(cls):
        pass
    @p.setter
    def p(cls):
        pass
'''
        assert_raises_with_msg(TypeError, 'Setters are not supported.',
                               exec, code, globals())
        assert_raises_with_msg(TypeError, 'Setters are not supported.',
                               classproperty, lambda c: None, lambda c, v: None)

    def test_cannot_have_deleter(self):
        code = '''
class Class:
    @classproperty
    def p(cls):
        pass
    @p.deleter
    def p(cls):
        pass
'''
        assert_raises_with_msg(TypeError, 'Deleters are not supported.',
                               exec, code, globals())
        assert_raises_with_msg(TypeError, 'Deleters are not supported.',
                               classproperty, lambda c: None, None, lambda c, v: None)

    def test_doc(self):
        class Class(self.cls):
            @classproperty
            def p(cls):
                """Doc for p."""
            q = classproperty(lambda cls: None, doc='Doc for q.')
        assert_equal(Class.__dict__['p'].__doc__, 'Doc for p.')
        assert_equal(Class.__dict__['q'].__doc__, 'Doc for q.')


if __name__ == "__main__":
    unittest.main()



import unittest
import warnings
from pathlib import Path

from robot.model import TestSuite
from robot.running import TestSuite as RunningTestSuite
from robot.result import TestSuite as ResultTestSuite
from robot.utils.asserts import (assert_equal, assert_true, assert_raises,
                                 assert_raises_with_msg)


class TestTestSuite(unittest.TestCase):

    def setUp(self):
        self.suite = TestSuite(metadata={'M': 'V'})

    def test_modify_medatata(self):
        self.suite.metadata['m'] = 'v'
        self.suite.metadata['n'] = 'w'
        assert_equal(dict(self.suite.metadata), {'M': 'v', 'n': 'w'})

    def test_set_metadata(self):
        self.suite.metadata = {'a': '1', 'b': '1'}
        self.suite.metadata['A'] = '2'
        assert_equal(dict(self.suite.metadata), {'a': '2', 'b': '1'})

    def test_create_and_add_suite(self):
        s1 = self.suite.suites.create(name='s1')
        s2 = TestSuite(name='s2')
        self.suite.suites.append(s2)
        assert_true(s1.parent is self.suite)
        assert_true(s2.parent is self.suite)
        assert_equal(list(self.suite.suites), [s1, s2])

    def test_reset_suites(self):
        s1 = TestSuite(name='s1')
        self.suite.suites = [s1]
        s2 = self.suite.suites.create(name='s2')
        assert_true(s1.parent is self.suite)
        assert_true(s2.parent is self.suite)
        assert_equal(list(self.suite.suites), [s1, s2])

    def test_name_from_source(self):
        for inp, exp in [(None, ''), ('', ''), ('name', 'Name'), ('name.robot', 'Name'),
                         ('naMe', 'naMe'), ('na_me', 'Na Me'), ('na_M_e_', 'na M e'),
                         ('prefix__name', 'Name'), ('__n', 'N'), ('naMe__', 'naMe')]:
            assert_equal(TestSuite.name_from_source(inp), exp)
            suite = TestSuite(source=inp)
            assert_equal(suite.name, exp)
            suite.suites.create(name='xxx')
            assert_equal(suite.name, exp or 'xxx')
            suite.name = 'new name'
            assert_equal(suite.name, 'new name')
            if inp:
                assert_equal(TestSuite(source=Path(inp)).name, exp)
                assert_equal(TestSuite(source=Path(inp).absolute()).name, exp)

    def test_name_from_source_with_extensions(self):
        for ext, exp in [('z', 'X.Y'), ('.z', 'X.Y'), ('Z', 'X.Y'), ('y.z', 'X'),
                         ('Y.z', 'X'), (['x', 'y', 'z'], 'X.Y')]:
            assert_equal(TestSuite.name_from_source('x.y.z', ext), exp)
            assert_equal(TestSuite.name_from_source('X.Y.Z', ext), exp)

    def test_name_from_source_with_bad_extensions(self):
        assert_raises_with_msg(
            ValueError,
            "File 'x.y' does not have extension 'z'.",
            TestSuite.name_from_source, 'x.y', extension='z'
        )
        assert_raises_with_msg(
            ValueError,
            "File 'x.y' does not have extension 'a', 'b' or 'c'.",
            TestSuite.name_from_source, 'x.y', ('a', 'b', 'c')
        )

    def test_suite_name_from_child_suites(self):
        suite = TestSuite()
        assert_equal(suite.name, '')
        assert_equal(suite.suites.create(name='foo').name, 'foo')
        assert_equal(suite.suites.create(name='bar').name, 'bar')
        assert_equal(suite.name, 'foo & bar')
        assert_equal(suite.suites.create(name='zap').name, 'zap')
        assert_equal(suite.name, 'foo & bar & zap')
        suite.name = 'new name'
        assert_equal(suite.name, 'new name')

    def test_nested_subsuites(self):
        suite = TestSuite(name='top')
        sub1 = suite.suites.create(name='sub1')
        sub2 = sub1.suites.create(name='sub2')
        assert_equal(list(suite.suites), [sub1])
        assert_equal(list(sub1.suites), [sub2])

    def test_adjust_source(self):
        absolute = Path('.').absolute()
        suite = TestSuite(source='dir')
        suite.suites = [TestSuite(source='dir/x.robot'),
                        TestSuite(source='dir/y.robot')]
        assert_equal(suite.source, Path('dir'))
        assert_equal(suite.suites[0].source, Path('dir/x.robot'))
        assert_equal(suite.suites[1].source, Path('dir/y.robot'))
        suite.adjust_source(root=absolute)
        assert_equal(suite.source, absolute / 'dir')
        assert_equal(suite.suites[0].source, absolute / 'dir/x.robot')
        assert_equal(suite.suites[1].source, absolute / 'dir/y.robot')
        suite.adjust_source(relative_to=absolute)
        assert_equal(suite.source, Path('dir'))
        assert_equal(suite.suites[0].source, Path('dir/x.robot'))
        assert_equal(suite.suites[1].source, Path('dir/y.robot'))
        suite.adjust_source(root='relative')
        assert_equal(suite.source, Path('relative/dir'))
        assert_equal(suite.suites[0].source, Path('relative/dir/x.robot'))
        assert_equal(suite.suites[1].source, Path('relative/dir/y.robot'))
        suite.adjust_source(relative_to='relative/dir', root=str(absolute))
        assert_equal(suite.source, absolute)
        assert_equal(suite.suites[0].source, absolute / 'x.robot')
        assert_equal(suite.suites[1].source, absolute / 'y.robot')

    def test_adjust_source_failures(self):
        absolute = Path('x.robot').absolute()
        assert_raises_with_msg(
            ValueError, 'Suite has no source.',
            TestSuite().adjust_source
        )
        assert_raises_with_msg(
            ValueError, f"Cannot set root for absolute source '{absolute}'.",
            TestSuite(source=absolute).adjust_source, root='whatever'
        )
        assert_raises(
            ValueError,
            TestSuite(source=absolute).adjust_source, relative_to='relative'
        )
        assert_raises(
            ValueError,
            TestSuite(source='relative').adjust_source, relative_to=absolute,
        )

    def test_set_tags(self):
        suite = TestSuite()
        suite.tests.create()
        suite.tests.create(tags=['t1', 't2'])
        suite.set_tags(add='a', remove=['t2', 'nonex'])
        suite.tests.create()
        assert_equal(list(suite.tests[0].tags), ['a'])
        assert_equal(list(suite.tests[1].tags), ['a', 't1'])
        assert_equal(list(suite.tests[2].tags), [])

    def test_set_tags_also_to_new_child(self):
        suite = TestSuite()
        suite.tests.create()
        suite.set_tags(add='a', remove=['t2', 'nonex'], persist=True)
        suite.tests.create(tags=['t1', 't2'])
        suite.tests = list(suite.tests)
        suite.tests.create()
        suite.suites.create().tests.create()
        assert_equal(list(suite.tests[0].tags), ['a'])
        assert_equal(list(suite.tests[1].tags), ['a', 't1'])
        assert_equal(list(suite.tests[2].tags), ['a'])
        assert_equal(list(suite.suites[0].tests[0].tags), ['a'])

    def test_all_tests_and_test_count(self):
        root = TestSuite()
        assert_equal(root.has_tests, False)
        assert_equal(root.test_count, 0)
        assert_equal(list(root.all_tests), [])
        for i in range(10):
            suite = root.suites.create()
            for j in range(100):
                suite.tests.create()
        assert_equal(root.has_tests, True)
        assert_equal(root.test_count, 1000)
        assert_equal(len(list(root.all_tests)), 1000)
        for suite in root.suites:
            assert_equal(suite.has_tests, True)
            assert_equal(list(suite.all_tests), list(suite.tests))

    def test_configure_only_works_with_root_suite(self):
        for Suite in TestSuite, RunningTestSuite, ResultTestSuite:
            root = Suite()
            child = root.suites.create()
            child.tests.create()
            root.configure(name='Configured')
            assert_equal(root.name, 'Configured')
            assert_raises_with_msg(
                ValueError, "'TestSuite.configure()' can only be used with "
                "the root test suite.", child.configure, name='Bang'
            )

    def test_slots(self):
        assert_raises(AttributeError, setattr, self.suite, 'attr', 'value')

    def test_keywords_deprecation(self):
        self.suite.setup.config(name='S')
        with warnings.catch_warnings(record=True) as w:
            kws = self.suite.keywords
            assert_equal(len(kws), 1)
            assert_true('deprecated' in str(w[0].message))
        assert_raises(AttributeError, kws.extend, ())
        assert_raises(AttributeError, setattr, self.suite, 'keywords', [])


class TestSuiteId(unittest.TestCase):

    def test_one_suite(self):
        assert_equal(TestSuite().id, 's1')

    def test_sub_suites(self):
        parent = TestSuite()
        for i in range(10):
            assert_equal(parent.suites.create().id, 's1-s%s' % (i+1))
        assert_equal(parent.suites[-1].suites.create().id, 's1-s10-s1')

    def test_id_is_dynamic(self):
        suite = TestSuite()
        sub = suite.suites.create().suites.create()
        assert_equal(sub.id, 's1-s1-s1')
        suite.suites = [sub]
        assert_equal(sub.id, 's1-s1')


class TestStringRepresentation(unittest.TestCase):

    def setUp(self):
        self.empty = TestSuite()
        self.ascii = TestSuite(name='Kekkonen')
        self.non_ascii = TestSuite(name='hyvä nimi')

    def test_str(self):
        for tc, expected in [(self.empty, ''),
                             (self.ascii, 'Kekkonen'),
                             (self.non_ascii, 'hyvä nimi')]:
            assert_equal(str(tc), expected)

    def test_repr(self):
        for tc, expected in [(self.empty, "TestSuite(name='')"),
                             (self.ascii, "TestSuite(name='Kekkonen')"),
                             (self.non_ascii, "TestSuite(name='hyvä nimi')")]:
            assert_equal(repr(tc), 'robot.model.' + expected)


if __name__ == '__main__':
    unittest.main()



import sys


ROBOT_LISTENER_API_VERSION = 2


def start_keyword(name, attrs):
    sys.stdout.write('start keyword %s\n' % name)
    sys.stderr.write('start keyword %s\n' % name)


def end_keyword(name, attrs):
    sys.stdout.write('end keyword %s\n' % name)
    sys.stderr.write('end keyword %s\n' % name)



import time
import unittest

from robot.utils.asserts import assert_equal
from robot.utils.robottime import TimestampCache


class FakeTimestampCache(TimestampCache):

    def __init__(self, epoch):
        TimestampCache.__init__(self)
        self.epoch = epoch + self.timezone_correction()

    def _get_epoch(self):
        return self.epoch

    def timezone_correction(self):
        dst = 3600 if time.daylight == 0 else 0
        tz = 7200 + time.timezone
        return (tz + dst)


@unittest.skip('skip for wrong timestamps for timezone, these are removed in the next version')
class TestTimestamp(unittest.TestCase):

    def test_new_timestamp(self):
        actual = FakeTimestampCache(1338816626.999).get_timestamp()
        assert_equal(actual, '20120604 16:30:26.999')

    def test_cached(self):
        cache = FakeTimestampCache(1338816626.900)
        cache.get_timestamp()
        cache.epoch += 0.099
        assert_equal(cache.get_timestamp(), '20120604 16:30:26.999')

    def test_round_to_next_second(self):
        cache = FakeTimestampCache(1338816626.0)
        assert_equal(cache.get_timestamp(), '20120604 16:30:26.000')
        cache.epoch += 0.9995
        assert_equal(cache.get_timestamp(), '20120604 16:30:27.000')

    def test_cache_timestamp_without_millis_separator(self):
        cache = FakeTimestampCache(1338816626.0)
        assert_equal(cache.get_timestamp(millissep=None), '20120604 16:30:26')
        assert_equal(cache.get_timestamp(millissep=None), '20120604 16:30:26')
        assert_equal(cache.get_timestamp(), '20120604 16:30:26.000')

    def test_separators(self):
        cache = FakeTimestampCache(1338816626.001)
        assert_equal(cache.get_timestamp(daysep='-', daytimesep='T'),
                      '2012-06-04T16:30:26.001')
        assert_equal(cache.get_timestamp(timesep='', millissep='X'),
                      '20120604 163026X001')


if __name__ == "__main__":
    unittest.main()



import unittest

from robot.utils.asserts import (assert_equal, assert_false, assert_true,
                                 assert_raises, assert_raises_with_msg)


from robot.utils import ConnectionCache


class ConnectionMock:

    def __init__(self, id=None):
        self.id = id
        self.closed_by_close = False
        self.closed_by_exit = False

    def close(self):
        self.closed_by_close = True

    def exit(self):
        self.closed_by_exit = True

    def __eq__(self, other):
        return hasattr(other, 'id') and self.id == other.id


class TestConnectionCache(unittest.TestCase):

    def setUp(self):
        self.cache = ConnectionCache()

    def test_initial(self):
        self._verify_initial_state()

    def test_no_connection(self):
        assert_raises_with_msg(RuntimeError, 'No open connection.', getattr,
                               ConnectionCache().current, 'whatever')
        assert_raises_with_msg(RuntimeError, 'Custom msg', getattr,
                               ConnectionCache('Custom msg').current, 'xxx')

    def test_register_one(self):
        conn = ConnectionMock()
        index = self.cache.register(conn)
        assert_equal(index, 1)
        assert_equal(self.cache.current, conn)
        assert_equal(self.cache.current_index, 1)
        assert_equal(self.cache._connections, [conn])
        assert_equal(self.cache._aliases, {})

    def test_register_multiple(self):
        conns = [ConnectionMock(1), ConnectionMock(2), ConnectionMock(3)]
        for i, conn in enumerate(conns):
            index = self.cache.register(conn)
            assert_equal(index, i+1)
            assert_equal(self.cache.current, conn)
            assert_equal(self.cache.current_index, i+1)
        assert_equal(self.cache._connections, conns)

    def test_register_multiple_equal_objects(self):
        conns = [ConnectionMock(1), ConnectionMock(1), ConnectionMock(1)]
        for i, conn in enumerate(conns):
            index = self.cache.register(conn)
            assert_equal(index, i+1)
            assert_equal(self.cache.current, conn)
            assert_equal(self.cache.current_index, i+1)
        assert_equal(self.cache._connections, conns)

    def test_register_multiple_same_object(self):
        conns = [ConnectionMock()] * 3
        for i, conn in enumerate(conns):
            index = self.cache.register(conn)
            assert_equal(index, i+1)
            assert_equal(self.cache.current, conn)
            assert_equal(self.cache.current_index, 1)
        assert_equal(self.cache._connections, conns)

    def test_set_current_index(self):
        self.cache.current_index = None
        assert_equal(self.cache.current_index, None)
        self._register('a', 'b')
        self.cache.current_index = 1
        assert_equal(self.cache.current_index, 1)
        assert_equal(self.cache.current.id, 'a')
        self.cache.current_index = None
        assert_equal(self.cache.current_index, None)
        assert_equal(self.cache.current, self.cache._no_current)
        self.cache.current_index = 2
        assert_equal(self.cache.current_index, 2)
        assert_equal(self.cache.current.id, 'b')

    def test_set_invalid_index(self):
        assert_raises(IndexError, setattr, self.cache, 'current_index', 1)

    def test_switch_with_index(self):
        self._register('a', 'b', 'c')
        self._assert_current('c', 3)
        self.cache.switch(1)
        self._assert_current('a', 1)
        self.cache.switch('2')
        self._assert_current('b', 2)

    def _assert_current(self, id, index):
        assert_equal(self.cache.current.id, id)
        assert_equal(self.cache.current_index, index)

    def test_switch_with_non_existing_index(self):
        self._register('a', 'b')
        assert_raises_with_msg(RuntimeError, "Non-existing index or alias '3'.",
                               self.cache.switch, 3)
        assert_raises_with_msg(RuntimeError, "Non-existing index or alias '42'.",
                               self.cache.switch, 42)

    def test_register_with_alias(self):
        conn = ConnectionMock()
        index = self.cache.register(conn, 'My Connection')
        assert_equal(index, 1)
        assert_equal(self.cache.current, conn)
        assert_equal(self.cache._connections, [conn])
        assert_equal(self.cache._aliases, {'myconnection': 1})

    def test_register_multiple_with_alias(self):
        c1 = ConnectionMock(); c2 = ConnectionMock(); c3 = ConnectionMock()
        for i, conn in enumerate([c1,c2,c3]):
            index = self.cache.register(conn, 'c%d' % (i+1))
            assert_equal(index, i+1)
            assert_equal(self.cache.current, conn)
        assert_equal(self.cache._connections, [c1, c2, c3])
        assert_equal(self.cache._aliases, {'c1': 1, 'c2': 2, 'c3': 3})

    def test_switch_with_alias(self):
        self._register('a', 'b', 'c', 'd', 'e')
        assert_equal(self.cache.current.id, 'e')
        self.cache.switch('a')
        assert_equal(self.cache.current.id, 'a')
        self.cache.switch('C')
        assert_equal(self.cache.current.id, 'c')
        self.cache.switch('  B   ')
        assert_equal(self.cache.current.id, 'b')

    def test_switch_with_non_existing_alias(self):
        self._register('a', 'b')
        assert_raises_with_msg(RuntimeError,
                               "Non-existing index or alias 'whatever'.",
                               self.cache.switch, 'whatever')

    def test_switch_with_alias_overriding_index(self):
        self._register('2', '1')
        self.cache.switch(1)
        assert_equal(self.cache.current.id, '2')
        self.cache.switch('1')
        assert_equal(self.cache.current.id, '1')

    def test_get_connection_with_index(self):
        self._register('a', 'b')
        assert_equal(self.cache.get_connection(1).id, 'a')
        assert_equal(self.cache.current.id, 'b')
        assert_equal(self.cache[2].id, 'b')

    def test_get_connection_with_alias(self):
        self._register('a', 'b')
        assert_equal(self.cache.get_connection('a').id, 'a')
        assert_equal(self.cache.current.id, 'b')
        assert_equal(self.cache['b'].id, 'b')

    def test_get_connection_with_none_returns_current(self):
        self._register('a', 'b')
        assert_equal(self.cache.get_connection().id, 'b')
        assert_equal(self.cache[None].id, 'b')

    def test_get_connection_with_none_fails_if_no_current(self):
        assert_raises_with_msg(RuntimeError,
                               'No open connection.',
                               self.cache.get_connection)

    def test_close_all(self):
        connections = self._register('a', 'b', 'c', 'd')
        self.cache.close_all()
        self._verify_initial_state()
        for conn in connections:
            assert_true(conn.closed_by_close)

    def test_close_all_with_given_method(self):
        connections = self._register('a', 'b', 'c', 'd')
        self.cache.close_all('exit')
        self._verify_initial_state()
        for conn in connections:
            assert_true(conn.closed_by_exit)

    def test_empty_cache(self):
        connections = self._register('a', 'b', 'c', 'd')
        self.cache.empty_cache()
        self._verify_initial_state()
        for conn in connections:
            assert_false(conn.closed_by_close)
            assert_false(conn.closed_by_exit)

    def test_iter(self):
        conns = ['a', object(), 1, None]
        for c in conns:
            self.cache.register(c)
        assert_equal(list(self.cache), conns)

    def test_len(self):
        assert_equal(len(self.cache), 0)
        self.cache.register(None)
        assert_equal(len(self.cache), 1)
        self.cache.register(None)
        assert_equal(len(self.cache), 2)
        self.cache.empty_cache()
        assert_equal(len(self.cache), 0)

    def test_truthy(self):
        assert_false(self.cache)
        self.cache.register(None)
        assert_true(self.cache)
        self.cache.current_index = None
        assert_false(self.cache)
        self.cache.current_index = 1
        assert_true(self.cache)
        self.cache.empty_cache()
        assert_false(self.cache)

    def test_resolve_alias_or_index(self):
        self.cache.register(ConnectionMock(), 'alias')
        assert_equal(self.cache.resolve_alias_or_index('alias'), 1)
        assert_equal(self.cache.resolve_alias_or_index('1'), 1)
        assert_equal(self.cache.resolve_alias_or_index(1), 1)

    def test_resolve_invalid_alias_or_index(self):
        assert_raises_with_msg(ValueError,
                               "Non-existing index or alias 'nonex'.",
                               self.cache.resolve_alias_or_index, 'nonex')
        assert_raises_with_msg(ValueError,
                               "Non-existing index or alias '1'.",
                               self.cache.resolve_alias_or_index, '1')
        assert_raises_with_msg(ValueError,
                               "Non-existing index or alias '42'.",
                               self.cache.resolve_alias_or_index, 42)

    def _verify_initial_state(self):
        assert_equal(self.cache.current, self.cache._no_current)
        assert_equal(self.cache.current_index, None)
        assert_equal(self.cache._connections, [])
        assert_equal(self.cache._aliases, {})

    def _register(self, *ids):
        connections = []
        for id in ids:
            conn = ConnectionMock(id)
            self.cache.register(conn, id)
            connections.append(conn)
        return connections


if __name__ == '__main__':
    unittest.main()



class _BaseLib:

    def __init__(self):
        self.registered = set()

    def register(self, name):
        self.registered.add(name)

    def should_be_registered(self, *expected):
        if self.registered != set(expected):
            raise AssertionError('Wrong registered: %s != %s'
                                 % (sorted(self.registered), sorted(expected)))


class Global(_BaseLib):
    ROBOT_LIBRARY_SCOPE = 'global'
    initializations = 0

    def __init__(self):
        Global.initializations += 1
        _BaseLib.__init__(self)

    def should_be_registered(self, *expected):
        if self.initializations != 1:
            raise AssertionError("Global library initialized more than once.")
        _BaseLib.should_be_registered(self, *expected)


class Suite(_BaseLib):
    ROBOT_LIBRARY_SCOPE = 'SUITE'


class TestSuite(_BaseLib):
    ROBOT_LIBRARY_SCOPE = 'TEST_SUITE'


class Test(_BaseLib):
    ROBOT_LIBRARY_SCOPE = 'TeSt'


class TestCase(_BaseLib):
    ROBOT_LIBRARY_SCOPE = 'TeSt CAse'


class Task(_BaseLib):
    # Any non-recognized value is mapped to TEST scope.
    ROBOT_LIBRARY_SCOPE = 'TASK'


class InvalidValue(_BaseLib):
    ROBOT_LIBRARY_SCOPE = 'invalid'


class InvalidEmpty(_BaseLib):
    pass


class InvalidMethod(_BaseLib):
    def ROBOT_LIBRARY_SCOPE(self):
        pass


class InvalidNone(_BaseLib):
    ROBOT_LIBRARY_SCOPE = None
